package main

import (
	"context"
	"net"
	"net/http"
	"net/url"
	"time"

	"github.com/camunda-cloud/zeebe/clients/go/pkg/worker"
	"github.com/camunda-cloud/zeebe/clients/go/pkg/zbc"
	"github.com/cenkalti/backoff"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/connections"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/data"
	"gitlab.mashroom.com/mashroom/backend/account/internal/service"
	"gitlab.mashroom.com/mashroom/backend/account/version"
	closecom "gitlab.mashroom.com/mashroom/backend/closecom/api"
	"gitlab.mashroom.com/mashroom/backend/common/api/health"
	"gitlab.mashroom.com/mashroom/backend/common/generic/addr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/bpms"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/error/mongoerr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/grpc"
	logger "gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/common/generic/metrics"
	"gitlab.mashroom.com/mashroom/backend/common/generic/tracer"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	listing "gitlab.mashroom.com/mashroom/backend/listing/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	"gitlab.mashroom.com/mashroom/backend/offer/api/rto"
	viewing "gitlab.mashroom.com/mashroom/backend/viewing/api"
)

var (
	envMongoURL      = env.ResolveString("MONGO_URL", "mongodb:27017")
	envMongoUsername = env.ResolveString("MONGO_USERNAME", "account-data")
	envMongoPassword = env.ResolveString("MONGO_PASSWORD", "account-data")
	envMongoAuthDB   = env.ResolveString("MONGO_AUTH", "admin")
	envMongoDB       = env.ResolveString("MONGO_DATABASE", "account-data")
	envMongoSSLMode  = env.ResolveBool("MONGO_SSL_MODE", false)
)

var (
	envWistiaMediaURL = env.ResolveString(
		"WISTIA_MEDIA_URL",
		"https://api.wistia.com/v1/medias.json",
	)
	envWistiaWebinarsProjectID = env.ResolveInt("WISTIA_WEBINARS_PROJECT_ID", 5756854)
	envWistiaHowToProjectID    = env.ResolveInt("WISTIA_HOW_TO_PROJECT_ID", 5807964)
	envWistiaAccessToken       = env.ResolveString(
		"WISTIA_ACCESS_TOKEN",
		"89fc357f59a706435afaced81116d2568842f1ca8e5d71d781d73b1b16336cdb",
	)
)

var (
	connectInvitationService = invitation.NewInvitationServiceClient(grpc.Dial(
		env.ResolveString("INVITATION_SERVICE_URL", "mashroom-invitation:10000"),
		"Invitation",
	))

	connectListingService = listing.NewListingServiceClient(grpc.Dial(
		env.ResolveString("LISTING_SERVICE_URL", "mashroom-listing:10000"),
		"Listing",
	))

	offerServiceURL     = env.ResolveString("OFFER_SERVICE_URL", "mashroom-offer:10000")
	connectOfferService = offer.NewOfferServiceClient(grpc.Dial(offerServiceURL, "Offer"))
	rtoServiceClient    = rto.NewRtoServiceClient(grpc.Dial(offerServiceURL, "Rto"))

	connectNotificationService = notification.NewNotificationServiceClient(grpc.Dial(
		env.ResolveString("NOTIFICATION_SERVICE_URL", "mashroom-notification:10000"),
		"Notification",
	))

	connectDocumentService = document.NewDocumentServiceClient(grpc.Dial(
		env.ResolveString("DOCUMENT_SERVICE_URL", "mashroom-document:10000"),
		"Document",
	))

	connectClosecomService = closecom.NewClosecomServiceClient(grpc.Dial(
		env.ResolveString("CLOSECOM_SERVICE_URL", "mashroom-closecom:10000"),
		"Closecom",
	))

	connectViewingService = viewing.NewViewingServiceClient(grpc.Dial(
		env.ResolveString("VIEWING_SERVICE_URL", "mashroom-viewing:10000"),
		"Viewing",
	))
)

func main() {
	ctx := context.Background()

	defer tracer.Close()

	repository, err := mongo.NewRepository(ctx, &mongo.Config{
		URL:          envMongoURL,
		Username:     envMongoUsername,
		Password:     envMongoPassword,
		Database:     envMongoDB,
		AuthSource:   envMongoAuthDB,
		MongoSSLMode: envMongoSSLMode,
	})
	if err != nil {
		panic(err)
	}

	conn := connections.Connections{
		Invitation:   connectInvitationService,
		Listing:      connectListingService,
		Offer:        connectOfferService,
		Rto:          rtoServiceClient,
		Notification: connectNotificationService,
		Document:     connectDocumentService,
		Closecom:     connectClosecomService,
		Viewing:      connectViewingService,
	}

	err = repository.MigrateSchema(ctx)
	if err != nil {
		panic(err)
	}

	go repository.MigrateData(
		ctx,
		backoff.NewConstantBackOff(time.Minute),
		data.Migrations(&conn)...,
	)

	wistiaURL, err := url.Parse(envWistiaMediaURL)
	if err != nil {
		panic(err)
	}

	var zbcClient zbc.Client

	if bpms.URL != "" {
		zbcClient, err = zbc.NewClient(&zbc.ClientConfig{
			UsePlaintextConnection: true,
			GatewayAddress:         bpms.URL,
		})
		if err != nil {
			panic(err)
		}
	}

	srv, err := service.NewService(&service.Config{
		Repository:  repository,
		Connections: conn,
		Wistia: service.Wistia{
			URL:               wistiaURL,
			AccessToken:       envWistiaAccessToken,
			WebinarsProjectID: int32(envWistiaWebinarsProjectID),
			HowToProjectID:    int32(envWistiaHowToProjectID),
			HTTPClient:        &http.Client{},
		},
		ZeebeClient: zbcClient,
	})
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("account", grpc.ServerOptionErrorTranslator(
		mongoerr.Translate,
	))
	if err != nil {
		panic(err)
	}

	account.RegisterAccountServiceServer(grpcServer, srv)

	logger.Infoln("account started")

	go func() {
		_, err = srv.UpdateWistiaCollection(ctx, &account.UpdateWistiaCollectionRequest{})
		if err != nil {
			logger.WithError(err).Error("Updating wistia collection")
		}
	}()

	healthServer := version.NewHealthServer()
	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go func() {
		_ = metrics.ConfigureListenAndServe(grpcServer)
	}()

	go func() {
		var w worker.JobWorker

		w, err = srv.RegisterProcessHandlers()
		if err != nil {
			panic(err)
		}

		go func() {
			<-ctx.Done()

			w.Close()
		}()

		if w != nil {
			w.AwaitClose()
		}
	}()

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
