// Package model provides model abstraction for account service
package model

import (
	"time"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
)

// AccountLogAction action type
type AccountLogAction string

// action types
const (
	AccountLogActionDelete  AccountLogAction = "delete"
	AccountLogActionRestore AccountLogAction = "restore"
)

// AccountDetails user details
type AccountDetails struct {
	FirstName string                `bson:"first_name"`
	LastName  string                `bson:"last_name"`
	FullName  string                `bson:"full_name"`
	Gender    account.AccountGender `bson:"gender"`
	UkNi      string                `bson:"uk_ni"`
	Birthday  time.Time             `bson:"birthday"`
	Avatar    string                `bson:"avatar"`
}

// GpsCoordinates location coordinates
type GpsCoordinates struct {
	Longitude float64 `bson:"longitude"`
	Latitude  float64 `bson:"latitude"`
}

// AccountLocation user address
type AccountLocation struct {
	Postcode string         `bson:"postcode"`
	Address1 string         `bson:"address1"`
	Address2 string         `bson:"address2"`
	Address3 string         `bson:"address3"`
	Address4 string         `bson:"address4"`
	Flat     string         `bson:"flat"`
	Building string         `bson:"building"`
	Street   string         `bson:"street"`
	City     string         `bson:"city"`
	GPS      GpsCoordinates `bson:"gps"`
	Country  string         `bson:"country"`
}

// Phone contact details
type Phone struct {
	Number string                   `bson:"number"`
	Type   account.AccountPhoneType `bson:"type"`
}

// AccountContactStatus verification status of user contacts
type AccountContactStatus struct {
	EmailVerified bool `bson:"email_verified"`
	PhoneVerified bool `bson:"phone_verified"`
}

// AccountContact user contact details
type AccountContact struct {
	EmailPrimary    string               `bson:"email_primary"`
	PhonePrimary    string               `bson:"phone_primary"`
	PhonesSecondary []*Phone             `bson:"phones_secondary"`
	ContactStatus   AccountContactStatus `bson:"contact_status"`
}

// AccountBankDetails banking information
type AccountBankDetails struct {
	Society           string `bson:"society"`
	SortCode          string `bson:"sort_code"`
	AccountNumber     string `bson:"account_number"`
	AccountName       string `bson:"account_name"`
	PersonalReference string `bson:"personal_reference"`
}

// AccountResources account-related objects
type AccountResources struct {
	OwnedProperties []string `bson:"owned_properties"`
	ViewingRequests []string `bson:"viewing_requests"`
}

// AccountFavorites objects favorited by account
type AccountFavorites struct {
	Properties []string `bson:"properties"`
}

// AccountConfirmation admin confirmation
type AccountConfirmation struct {
	Text      string    `bson:"text"`
	CreatedAt time.Time `bson:"created_at"`
	Confirm   bool      `bson:"confirm"`
}

// AccountLog of global operations on account
type AccountLog struct {
	Action AccountLogAction `bson:"action"`
	Dt     time.Time        `bson:"dt"`
	User   string           `bson:"user"`
}

// Account model
type Account struct {
	ID                   string                      `bson:"_id,omitempty"`
	Location             AccountLocation             `bson:"location"`
	Details              AccountDetails              `bson:"details"`
	Contact              AccountContact              `bson:"contact"`
	Confirmation         AccountConfirmation         `bson:"confirmation"`
	BankDetails          AccountBankDetails          `bson:"bank_details"`
	Resources            AccountResources            `bson:"resources"`
	Favorites            AccountFavorites            `bson:"favorites"`
	Log                  []*AccountLog               `bson:"log"`
	ViewedVideosIDs      []int32                     `bson:"viewed_videos_ids"`
	CreatedAt            time.Time                   `bson:"created_at"`
	LastLogin            time.Time                   `bson:"last_login"`
	TosVersion           int64                       `bson:"tos_version"`
	CreationType         account.AccountCreationType `bson:"creation_type"`
	Admin                bool                        `bson:"admin"`
	Block                bool                        `bson:"block"`
	Deleted              bool                        `bson:"deleted"`
	Test                 bool                        `bson:"test"`
	EnableLandlordTool   bool                        `bson:"enable_landlord_tool"`
	EntryPoint           account.EntryPoint          `bson:"entry_point"`
	AuthorizationOrigin  account.AuthorizationOrigin `bson:"authorization_origin"`
	IdentityVerification bool                        `bson:"identity_verification"`
	Generated            bool                        `bson:"generated,omitempty"`
	IsVerified           bool                        `bson:"is_verified,omitempty"`
}

// AccountHistory model
type AccountHistory struct {
	ID                  string                      `bson:"_id,omitempty"`
	AccountID           string                      `bson:"account_id"`
	Action              string                      `bson:"action"`
	Location            AccountLocation             `bson:"location"`
	Details             AccountDetails              `bson:"details"`
	Contact             AccountContact              `bson:"contact"`
	Confirmation        AccountConfirmation         `bson:"confirmation"`
	BankDetails         AccountBankDetails          `bson:"bank_details"`
	Resources           AccountResources            `bson:"resources"`
	Favorites           AccountFavorites            `bson:"favorites"`
	Log                 []*AccountLog               `bson:"log"`
	ViewedVideosIDs     []int32                     `bson:"viewed_videos_ids"`
	CreatedAt           time.Time                   `bson:"created_at"`
	UpdatedAt           time.Time                   `bson:"updated_at,omitempty"`
	LastLogin           time.Time                   `bson:"last_login"`
	TosVersion          int64                       `bson:"tos_version"`
	CreationType        account.AccountCreationType `bson:"creation_type"`
	Admin               bool                        `bson:"admin"`
	Block               bool                        `bson:"block"`
	Deleted             bool                        `bson:"deleted"`
	Test                bool                        `bson:"test"`
	EnableLandlordTool  bool                        `bson:"enable_landlord_tool"`
	EntryPoint          account.EntryPoint          `bson:"entry_point"`
	AuthorizationOrigin account.AuthorizationOrigin `bson:"authorization_origin"`
}
