//go:generate mockgen -package=mock -destination=mock/repository.go . Repository

// Package model provides persistent repository models and abstraction interfaces
package model

import (
	"context"

	"github.com/cenkalti/backoff"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/generic/dbfilter"
	"gitlab.mashroom.com/mashroom/backend/common/generic/migrations"
)

// Repository provides persistent operations for account service
type Repository interface {
	MigrateSchema(ctx context.Context) error
	MigrateData(
		ctx context.Context,
		backoff backoff.BackOff,
		ms ...*migrations.DataMigration,
	)
	AccountCreate(ctx context.Context, acc *Account) error
	AccountUpdate(ctx context.Context, acc *Account) error
	AccountDelete(ctx context.Context, id string) error
	AccountCount(ctx context.Context, conditions ...dbfilter.Condition) (int64, error)
	AccountLookup(ctx context.Context, conditions ...dbfilter.Condition) (*Account, error)
	AccountUpdateByDecision(ctx context.Context, id string, acc *Account) (err error)
	AccountUpdateLastLogin(ctx context.Context, id string) error
	AccountUpdateBlock(ctx context.Context, id string, block bool) error
	AccountUpdateConfirmation(ctx context.Context, id string, confirm *AccountConfirmation) error
	AccountUpdateVerifiedStatus(ctx context.Context, id string) (err error)
	AccountGetByID(ctx context.Context, id string) (*Account, error)
	AccountGetByIDAdmin(ctx context.Context, id string, admin, soft bool) (*Account, error)
	AccountGetByAvatar(ctx context.Context, avatar string) (*Account, error)
	AccountGetByEmailAdmin(ctx context.Context, email string, admin bool) (*Account, error)
	AccountGetByPhoneAdmin(ctx context.Context, phone string, admin bool) (*Account, error)
	AccountListByFilter(ctx context.Context, params *query.QueryParams) ([]*Account, int64, error)
	AccountDeactivate(ctx context.Context, id, actor string) error
	AccountReactivate(ctx context.Context, id, actor string) error
	WistiaVideoSave(ctx context.Context, video *WistiaVideo) error
	WistiaVideoDelete(ctx context.Context, ids []int32, projectID int32) error
	WistiaVideoListByProjectID(ctx context.Context, projectID int32) ([]*WistiaVideo, error)
	VeriffSessionCreate(ctx context.Context, veriff *VeriffSession) error
	VeriffSessionUpdate(ctx context.Context, veriff *VeriffSession) error
	AdminVerifSessionUpdate(ctx context.Context, verif *VeriffSession) (err error)
	VeriffSessionUpdateFullName(ctx context.Context, id string, fullName string) (err error)
	SessionGetByAccountID(ctx context.Context, id string) (*VeriffSession, error)
	SessionGetByAccountAndSessionIDs(ctx context.Context,
		accountID string,
		sessionID string) (veriff *VeriffSession, err error)
	VeriffSessionUpdateByID(ctx context.Context, id string) (err error)
	VeriffRecordsCountByID(ctx context.Context, id string) (int64, error)
	VeriffListByAccountID(ctx context.Context, id string, params ...*query.QueryParams) ([]*VeriffSession, error)
	VeriffSessionUpdateByVendorData(ctx context.Context, veriff *VeriffSession, vendorData string) (err error)
	VeriffSessionUpdateCodeByVendorData(ctx context.Context, code int32, status, vendorData string) (err error)
	VeriffGetAccountIDByVendorData(ctx context.Context, vendorData string) (veriff *VeriffSession, err error)
	CheckApprovedStatusByAccountID(ctx context.Context, id string) (bool, error)
	CreateOwnershipVerification(ctx context.Context, ownership OwnershipVerification) error
	GetOwnershipVerificationByID(ctx context.Context, ownershipID string) (ownership *OwnershipVerification, err error)
	GetOwnershipVerificationByPropertyID(ctx context.Context,
		propertyID string) (ownership *OwnershipVerification, err error)
	UpdateOwnershipVerification(ctx context.Context, ownership *OwnershipVerification) (err error)
	GetOwnershipList(ctx context.Context, params ...*query.QueryParams) (ownerships []*OwnershipVerification, err error)
	GetOwnershipListByAccountID(ctx context.Context,
		accountID string,
		params ...*query.QueryParams,
	) (ownerships []*OwnershipVerification, err error)
	CheckOwnershipVerificationByPropertyID(ctx context.Context, propertyID string) (string, error)
}
