package mongo

import (
	"context"
	"time"

	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/generic/dbfilter"
	"gitlab.mashroom.com/mashroom/backend/common/generic/dbfilter/mongofilter"
	"gitlab.mashroom.com/mashroom/backend/common/generic/mongoquery"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

var caseInsensitive = options.FindOne().SetCollation(&options.Collation{Locale: "en", Strength: 2})

const (
	// CreateAction for history table
	CreateAction string = "create"
	// UpdateAction for history table
	UpdateAction string = "update"
)

func createHistoryModel(acc *model.Account, action string) *model.AccountHistory {
	return &model.AccountHistory{
		AccountID:           acc.ID,
		Action:              action,
		Location:            acc.Location,
		Details:             acc.Details,
		Contact:             acc.Contact,
		Confirmation:        acc.Confirmation,
		BankDetails:         acc.BankDetails,
		Resources:           acc.Resources,
		Favorites:           acc.Favorites,
		Log:                 acc.Log,
		ViewedVideosIDs:     acc.ViewedVideosIDs,
		CreatedAt:           acc.CreatedAt,
		LastLogin:           acc.LastLogin,
		TosVersion:          acc.TosVersion,
		CreationType:        acc.CreationType,
		Admin:               acc.Admin,
		Block:               acc.Block,
		Deleted:             acc.Deleted,
		Test:                acc.Test,
		EnableLandlordTool:  acc.EnableLandlordTool,
		EntryPoint:          acc.EntryPoint,
		AuthorizationOrigin: acc.AuthorizationOrigin,
	}
}

func (repo *repositoryImpl) AccountCreate(ctx context.Context, acc *model.Account) (err error) {
	acc.CreatedAt = time.Now()

	accountHistory := createHistoryModel(acc, CreateAction)
	accountHistory.CreatedAt = time.Now()

	_, err = repo.collectionAccounts.InsertOne(ctx, acc)
	if err != nil {
		return
	}

	_, err = repo.collectionAccountsHistory.InsertOne(ctx, accountHistory)

	return
}

func (repo *repositoryImpl) AccountUpdate(ctx context.Context, acc *model.Account) (err error) {
	_, err = repo.collectionAccounts.ReplaceOne(ctx, bson.M{"_id": acc.ID}, acc)
	if err != nil {
		return err
	}

	accountHistory := createHistoryModel(acc, UpdateAction)
	accountHistory.UpdatedAt = time.Now()

	_, err = repo.collectionAccountsHistory.InsertOne(ctx, accountHistory)

	return
}

func (repo *repositoryImpl) AccountUpdateByDecision(ctx context.Context, id string, acc *model.Account) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"is_verified":        acc.IsVerified,
			"details.first_name": acc.Details.FirstName,
			"details.last_name":  acc.Details.LastName,
			"details.gender":     acc.Details.Gender,
			"details.birthday":   acc.Details.Birthday,
		},
	})

	return
}

func (repo *repositoryImpl) AccountUpdateBlock(ctx context.Context, id string, block bool) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"block": block,
		},
	})

	return
}

func (repo *repositoryImpl) AccountUpdateConfirmation(
	ctx context.Context,
	id string,
	confirm *model.AccountConfirmation,
) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"confirmation": confirm,
		},
	})

	return
}

func (repo *repositoryImpl) AccountUpdateVerifiedStatus(ctx context.Context, id string) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"is_verified": true,
		},
	})

	return
}

func (repo *repositoryImpl) AccountDelete(ctx context.Context, id string) (err error) {
	_, err = repo.collectionAccounts.DeleteOne(ctx, bson.M{"_id": id})

	return
}

func (repo *repositoryImpl) AccountUpdateLastLogin(ctx context.Context, id string) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"last_login": time.Now(),
		},
	})

	return
}

func (repo *repositoryImpl) AccountGetByID(ctx context.Context, id string) (acc *model.Account, err error) {
	acc = &model.Account{}

	err = repo.collectionAccounts.FindOne(ctx, &bson.M{"_id": id}).Decode(acc)

	return
}

func (repo *repositoryImpl) AccountGetByIDAdmin(
	ctx context.Context,
	id string,
	admin, soft bool,
) (acc *model.Account, err error) {
	acc = &model.Account{}

	err = repo.collectionAccounts.FindOne(ctx, &bson.M{"_id": id, "admin": admin}).Decode(acc)
	if soft && err == mongo.ErrNoDocuments {
		acc = &model.Account{
			ID: "-",
			Details: model.AccountDetails{
				FirstName: "-",
			},
		}
		err = nil
	}

	return
}

func (repo *repositoryImpl) AccountGetByAvatar(ctx context.Context, avatar string) (acc *model.Account, err error) {
	acc = &model.Account{}

	err = repo.collectionAccounts.FindOne(ctx, &bson.M{"details.avatar": avatar}).Decode(acc)

	return
}

func (repo *repositoryImpl) AccountGetByEmailAdmin(
	ctx context.Context,
	email string,
	admin bool,
) (acc *model.Account, err error) {
	acc = &model.Account{}

	err = repo.collectionAccounts.FindOne(ctx, &bson.M{
		"contact.email_primary": email,
		"admin":                 admin,
	}, caseInsensitive).Decode(acc)

	return
}

func (repo *repositoryImpl) AccountGetByPhoneAdmin(
	ctx context.Context,
	phone string,
	admin bool,
) (acc *model.Account, err error) {
	acc = &model.Account{}

	err = repo.collectionAccounts.FindOne(ctx, &bson.M{
		"contact.phone_primary": phone,
		"admin":                 admin,
	}).Decode(acc)

	return
}

func (repo *repositoryImpl) AccountListByFilter(
	ctx context.Context,
	params *query.QueryParams,
) (accs []*model.Account, total int64, err error) {
	q, opt, err := mongoquery.BuildFilter(model.Account{}, params, nil)
	if err != nil {
		return
	}

	cur, err := repo.collectionAccounts.Find(ctx, q, opt)
	if err != nil {
		return
	}

	err = cur.All(ctx, &accs)
	if err != nil {
		return
	}

	total, err = repo.collectionAccounts.CountDocuments(ctx, q)

	return
}

func (repo *repositoryImpl) AccountDeactivate(ctx context.Context, id, actor string) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"deleted": true,
		},
		"$push": bson.M{
			"log": &model.AccountLog{
				Action: model.AccountLogActionDelete,
				Dt:     time.Now(),
				User:   actor,
			},
		},
	})

	return
}

func (repo *repositoryImpl) AccountReactivate(ctx context.Context, id, actor string) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"_id": id}, bson.M{
		"$set": bson.M{
			"deleted": false,
		},
		"$push": bson.M{
			"log": &model.AccountLog{
				Action: model.AccountLogActionRestore,
				Dt:     time.Now(),
				User:   actor,
			},
		},
	})

	return
}

func (repo *repositoryImpl) AccountCount(
	ctx context.Context,
	conditions ...dbfilter.Condition,
) (int64, error) {
	return mongofilter.Count(ctx, repo.collectionAccounts, dbfilter.And(conditions...))
}

func (repo *repositoryImpl) AccountLookup(
	ctx context.Context,
	conditions ...dbfilter.Condition,
) (*model.Account, error) {
	cur, err := mongofilter.Execute(
		ctx,
		repo.collectionAccounts,
		dbfilter.And(conditions...),
		options.Find().SetLimit(1),
	)
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = cur.Close(ctx)
	}()

	if !cur.Next(ctx) {
		return nil, nil
	}

	var m model.Account

	err = cur.Decode(&m)
	if err != nil {
		return nil, err
	}

	return &m, nil
}
