// Package mongo provides mongo repository implementation for account service
package mongo

import (
	"context"
	"crypto/tls"
	"fmt"

	"github.com/cenkalti/backoff"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	v1 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v1"
	v10 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v10"
	v11 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v11"
	v12 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v12"
	v13 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v13"
	v14 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v14"
	v15 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v15"
	v2 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v2"
	v3 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v3"
	v4 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v4"
	v5 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v5"
	v6 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v6"
	v7 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v7"
	v8 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v8"
	v9 "gitlab.mashroom.com/mashroom/backend/account/internal/model/mongo/migrations/schema/v9"
	"gitlab.mashroom.com/mashroom/backend/common/generic/migrations"
	migrationsMongo "gitlab.mashroom.com/mashroom/backend/common/generic/migrations/mongo"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

// collection names
const (
	CollectionAccounts        = "accounts"
	CollectionAccountsHistory = "accounts_history"
	CollectionWistia          = "wistia"
	CollectionVeriff          = "veriff"
	CollectionOwnership       = "ownership"
)

// Config mongo repository config
type Config struct {
	URL          string
	Username     string
	Password     string
	Database     string
	AuthSource   string
	MongoSSLMode bool
}

type repositoryImpl struct {
	client                    *mongo.Client
	database                  *mongo.Database
	collectionAccounts        *mongo.Collection
	collectionAccountsHistory *mongo.Collection
	collectionWistia          *mongo.Collection
	collectionVeriff          *mongo.Collection
	collectionOwnership       *mongo.Collection
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid repository config")
	}

	if config.URL == "" {
		return fmt.Errorf("empty url in repository config")
	}

	if config.Username == "" {
		return fmt.Errorf("empty username in repository config")
	}

	if config.Password == "" {
		return fmt.Errorf("empty password in repository config")
	}

	if config.Database == "" {
		return fmt.Errorf("empty database in repository config")
	}

	if config.AuthSource == "" {
		return fmt.Errorf("empty database in repository config")
	}

	return nil
}

// NewRepository returns new repository instance
func NewRepository(ctx context.Context, config *Config) (model.Repository, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	clientOptions := options.Client().ApplyURI(fmt.Sprintf("mongodb://%s", config.URL)).
		SetAuth(options.Credential{
			AuthSource:  config.AuthSource,
			Username:    config.Username,
			Password:    config.Password,
			PasswordSet: true,
		})

	if config.MongoSSLMode {
		clientOptions.SetTLSConfig(&tls.Config{})
	}

	client, err := mongo.NewClient(clientOptions)
	if err != nil {
		return nil, err
	}

	err = client.Connect(ctx)
	if err != nil {
		return nil, err
	}

	db := client.Database(config.Database)

	return &repositoryImpl{
		client:                    client,
		database:                  db,
		collectionAccounts:        db.Collection(CollectionAccounts),
		collectionAccountsHistory: db.Collection(CollectionAccountsHistory),
		collectionWistia:          db.Collection(CollectionWistia),
		collectionVeriff:          db.Collection(CollectionVeriff),
		collectionOwnership:       db.Collection(CollectionOwnership),
	}, nil
}

func (repo *repositoryImpl) MigrateSchema(ctx context.Context) error {
	err := migrations.MigrateSchema(migrationsMongo.WithInstance(ctx, repo.database),
		&migrationsMongo.Migration{
			Version: "v0",
			Up: func(ctx context.Context, db *mongo.Database, migrationID string) (int, error) {
				return -1, nil
			},
		},
		&migrationsMongo.Migration{
			Version: "v1",
			Up:      v1.Up,
		},
		&migrationsMongo.Migration{
			Version: "v2",
			Up:      v2.Up,
		},
		&migrationsMongo.Migration{
			Version: "v3",
			Up:      v3.Up,
		},
		&migrationsMongo.Migration{
			Version: "v4",
			Up:      v4.Up,
		},
		&migrationsMongo.Migration{
			Version: "v5",
			Up:      v5.Up,
		},
		&migrationsMongo.Migration{
			Version: "v6",
			Up:      v6.Up,
		},
		&migrationsMongo.Migration{
			Version: "v7",
			Up:      v7.Up,
		},
		&migrationsMongo.Migration{
			Version: "v8",
			Up:      v8.Up,
		},
		&migrationsMongo.Migration{
			Version: "v9",
			Up:      v9.Up,
		},
		&migrationsMongo.Migration{
			Version: "v10",
			Up:      v10.Up,
		},
		&migrationsMongo.Migration{
			Version: "v11",
			Up:      v11.Up,
		},
		&migrationsMongo.Migration{
			Version: "v12",
			Up:      v12.Up,
		},
		&migrationsMongo.Migration{
			Version: "v13",
			Up:      v13.Up,
		},
		&migrationsMongo.Migration{
			Version: "v14",
			Up:      v14.Up,
		},
		&migrationsMongo.Migration{
			Version: "v15",
			Up:      v15.Up,
		},
	)
	if err != nil {
		return err
	}

	return nil
}

func (repo *repositoryImpl) MigrateData(
	ctx context.Context,
	backoff backoff.BackOff,
	ms ...*migrations.DataMigration,
) {
	migrations.MigrateData(
		migrationsMongo.WithInstance(ctx, repo.database),
		backoff,
		ms...,
	)
}
