package d1v0000

import (
	"context"
	"errors"
	"strings"

	"gitlab.mashroom.com/mashroom/backend/account/internal/connections"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	closecom "gitlab.mashroom.com/mashroom/backend/closecom/api"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"gitlab.mashroom.com/mashroom/backend/common/generic/util"
	listing "gitlab.mashroom.com/mashroom/backend/listing/api"
	viewing "gitlab.mashroom.com/mashroom/backend/viewing/api"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func startClosecom(
	ctx context.Context,
	db *mongo.Database,
	conns *connections.Connections,
	migrationID string,
) (affected int, err error) {
	col := db.Collection("accounts")

	migrationIDParts := strings.Split(migrationID, ".")
	migrationIDPrefix := strings.Join(migrationIDParts[:len(migrationIDParts)-1], ".")

	cur, err := col.Find(ctx, bson.M{
		migrationID: bson.M{"$ne": true},
		"$or": bson.A{
			bson.M{migrationIDPrefix + ".attempt": bson.M{"$exists": false}},
			bson.M{migrationIDPrefix + ".attempt": bson.M{"$lt": 5}},
		},
	})
	if errors.Is(err, mongo.ErrNoDocuments) {
		return 0, nil
	}

	if err != nil {
		return 0, err
	}

	for cur.Next(ctx) {
		affected++

		var m model.Account

		err = cur.Decode(&m)
		if err != nil {
			continue
		}

		_, err = conns.Closecom.LeadContactSave(ctx, &closecom.LeadContactSaveRequest{
			UserId:     m.ID,
			Name:       util.JoinNonblankStrings(" ", m.Details.FirstName, m.Details.LastName),
			Email:      m.Contact.EmailPrimary,
			Phone:      m.Contact.PhonePrimary,
			EntryPoint: m.EntryPoint.String(),
			Signup:     timestamppb.New(m.CreatedAt),
		})
		if err != nil {
			log.WithError(err).WithField("user", m.ID).Error("exporting account as lead")

			continue
		}

		_, err = conns.Viewing.ServiceUserSync(ctx, &viewing.ServiceUserSyncRequest{
			UserId: m.ID,
		})
		if err != nil {
			log.WithError(err).WithField("user", m.ID).Error("triggering viewing export")
		}

		_, err = conns.Listing.ServiceUserSync(ctx, &listing.ServiceUserSyncRequest{
			UserId: m.ID,
		})
		if err != nil {
			log.WithError(err).WithField("user", m.ID).Error("triggering listing export")
		}

		_, err = col.UpdateOne(ctx, bson.M{"_id": m.ID},
			bson.M{"$set": bson.M{migrationID: true}, "$inc": bson.M{migrationIDPrefix + ".attempt": 1}},
		)
		if err != nil {
			log.WithError(err).WithField("user", m.ID).Error("error updating account record")
			return
		}
	}

	return
}
