package mongo

import (
	"context"

	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo/options"

	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
)

func (repo *repositoryImpl) CreateOwnershipVerification(
	ctx context.Context,
	ownership model.OwnershipVerification,
) error {
	_, err := repo.collectionOwnership.InsertOne(ctx, ownership)
	if err != nil {
		log.WithError(err).Errorf("CreateOwnershipVerification err: %s", err.Error())
		return err
	}

	return nil
}

func (repo *repositoryImpl) UpdateOwnershipVerification(
	ctx context.Context,
	ownership *model.OwnershipVerification,
) (err error) {
	_, err = repo.collectionOwnership.UpdateOne(ctx,
		bson.M{"account_id": ownership.AccountID,
			"ownership_id": ownership.OwnershipID},
		bson.M{
			"$set": bson.M{
				"first_name":     ownership.FirstName,
				"last_name":      ownership.LastName,
				"verified_by_id": ownership.VerifiedByID,
				"status":         ownership.Status,
				"is_company":     ownership.IsCompany,
				"is_purchased":   ownership.IsPurchased,
				"company_name":   ownership.CompanyName,
				"notes":          ownership.Notes,
				"document_ids":   ownership.DocumentIDs,
				"updated_at":     ownership.UpdatedAt,
			},
		})
	if err != nil {
		log.WithError(err).Errorf("UpdateOwnershipVerification err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) GetOwnershipVerificationByID(
	ctx context.Context,
	ownershipID string,
) (ownership *model.OwnershipVerification, err error) {
	ownership = &model.OwnershipVerification{}

	err = repo.collectionOwnership.FindOne(ctx, bson.M{
		"ownership_id": ownershipID,
	}).Decode(ownership)
	if err != nil {
		log.WithError(err).Errorf("GetOwnershipVerificationByID err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) GetOwnershipVerificationByPropertyID(
	ctx context.Context,
	propertyID string,
) (ownership *model.OwnershipVerification, err error) {
	ownership = &model.OwnershipVerification{}

	err = repo.collectionOwnership.FindOne(ctx, bson.M{
		"property_id": propertyID,
	}).Decode(ownership)
	if err != nil {
		log.WithError(err).Errorf("GetOwnershipVerificationByPropertyID err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) GetOwnershipList(
	ctx context.Context,
	params ...*query.QueryParams,
) (ownerships []*model.OwnershipVerification, err error) {
	cur, err := repo.collectionOwnership.Find(ctx, bson.M{})
	if err != nil {
		log.WithError(err).Errorf("GetOwnershipList err: %s", err.Error())
		return nil, err
	}

	err = cur.All(ctx, &ownerships)
	if err != nil {
		log.WithError(err).Errorf("GetOwnershipList err: %s", err.Error())
		return
	}

	return
}

func (repo *repositoryImpl) GetOwnershipListByAccountID(
	ctx context.Context,
	accountID string,
	params ...*query.QueryParams,
) (ownerships []*model.OwnershipVerification, err error) {
	cur, err := repo.collectionOwnership.Find(ctx, bson.M{
		"account_id": accountID,
	})
	if err != nil {
		log.WithError(err).Errorf("GetOwnershipListByAccountID err: %s", err.Error())
		return nil, err
	}

	err = cur.All(ctx, &ownerships)
	if err != nil {
		log.WithError(err).Errorf("GetOwnershipListByAccountID err: %s", err.Error())
		return
	}

	return
}

func (repo *repositoryImpl) CheckOwnershipVerificationByPropertyID(
	ctx context.Context,
	propertyID string,
) (string, error) {
	ownership := []model.OwnershipVerification{}

	opts := options.Find().SetSort(bson.D{{Key: "updated_at", Value: -1}}).SetLimit(1)

	cur, err := repo.collectionOwnership.Find(ctx, bson.M{"property_id": propertyID}, opts)
	if err != nil {
		log.WithError(err).Errorf("CheckOwnershipVerificationByPropertyID err: %s", err.Error())
		return "", err
	}

	err = cur.All(ctx, &ownership)
	if err != nil {
		log.WithError(err).Errorf("CheckOwnershipVerificationByPropertyID err: %s", err.Error())
		return "", err
	}

	if len(ownership) > 0 {
		for _, v := range ownership {
			return v.Status, nil
		}
	}

	return "", nil
}
