package mongo

import (
	"context"
	"time"

	"go.mongodb.org/mongo-driver/mongo"

	"gitlab.mashroom.com/mashroom/backend/common/generic/log"

	"go.mongodb.org/mongo-driver/mongo/options"

	"gitlab.mashroom.com/mashroom/backend/account/internal/service"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"

	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"go.mongodb.org/mongo-driver/bson"
)

func (repo *repositoryImpl) VeriffSessionCreate(ctx context.Context, veriff *model.VeriffSession) (err error) {
	veriff.CreatedAt = time.Now()
	veriff.UpdatedAt = time.Now()

	_, err = repo.collectionVeriff.InsertOne(ctx, veriff)
	if err != nil {
		log.WithError(err).Errorf("VeriffSessionCreate err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) VeriffSessionUpdate(ctx context.Context, veriff *model.VeriffSession) (err error) {
	_, err = repo.collectionVeriff.UpdateOne(ctx,
		bson.M{"account_id": veriff.AccountID,
			"status": service.VeriffStatusCreated},
		bson.M{
			"$set": bson.M{
				"status":      veriff.Status,
				"session_id":  veriff.SessionID,
				"reason":      veriff.Reason,
				"code":        veriff.Code,
				"reason_code": veriff.ReasonCode,
				"updated_at":  time.Now(),
				"is_admin":    veriff.IsAdmin,
				"admin_id":    veriff.AdminID,
			},
		})
	if err != nil {
		log.WithError(err).Errorf("VeriffSessionUpdate err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) AdminVerifSessionUpdate(ctx context.Context, verif *model.VeriffSession) (err error) {
	_, err = repo.collectionVeriff.UpdateOne(ctx,
		bson.M{"account_id": verif.AccountID,
			"session_id": verif.SessionID},
		bson.M{
			"$set": bson.M{
				"status":     verif.Status,
				"first_name": verif.FirstName,
				"last_name":  verif.LastName,
				"documents":  verif.Documents,
				"notes":      verif.Notes,
				"updated_at": time.Now(),
				"is_admin":   verif.IsAdmin,
			},
		})
	if err != nil {
		log.WithError(err).Errorf("AdminVerifSessionUpdate err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) VeriffSessionUpdateFullName(
	ctx context.Context,
	id string,
	fullName string,
) (err error) {
	_, err = repo.collectionAccounts.UpdateOne(ctx, bson.M{"session_id": id}, bson.M{
		"$set": bson.M{
			"full_name": fullName,
		},
	})
	if err != nil {
		log.WithError(err).Errorf("VeriffSessionUpdateFullName err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) VeriffSessionUpdateByID(ctx context.Context, id string) (err error) {
	_, err = repo.collectionVeriff.UpdateOne(ctx, bson.M{"session_id": id}, bson.M{
		"$set": bson.M{
			"status":     service.VeriffStatusAbandoned,
			"updated_at": time.Now(),
		},
	})
	if err != nil {
		log.WithError(err).Errorf("VeriffSessionUpdateByID err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) VeriffSessionUpdateByVendorData(ctx context.Context, veriff *model.VeriffSession,
	vendorData string) (err error) {
	_, err = repo.collectionVeriff.UpdateOne(ctx, bson.M{"vendor_data": vendorData}, bson.M{
		"$set": bson.M{
			"status":      veriff.Status,
			"reason":      veriff.Reason,
			"code":        veriff.Code,
			"reason_code": veriff.ReasonCode,
			"updated_at":  time.Now(),
		},
	})
	if err != nil {
		log.WithError(err).Errorf("VeriffSessionUpdateByID err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) VeriffSessionUpdateCodeByVendorData(ctx context.Context, code int32, status,
	vendorData string) (err error) {
	_, err = repo.collectionVeriff.UpdateOne(ctx, bson.M{"vendor_data": vendorData}, bson.M{
		"$set": bson.M{
			"status":     status,
			"code":       code,
			"updated_at": time.Now(),
		},
	})
	if err != nil {
		log.WithError(err).Errorf("VeriffSessionUpdateCodeByVendorData err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) SessionGetByAccountID(
	ctx context.Context,
	id string,
) (veriff *model.VeriffSession, err error) {
	veriff = &model.VeriffSession{}

	err = repo.collectionVeriff.FindOne(ctx, bson.M{
		"account_id": id,
		"status":     service.VeriffStatusCreated,
	}).Decode(veriff)
	if err == mongo.ErrNoDocuments {
		veriff = &model.VeriffSession{
			Status:    service.VeriffStatusUndefined,
			SessionID: "",
		}
		err = nil
	}

	if err != nil {
		log.WithError(err).Errorf("SessionGetByAccountID err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) SessionGetByAccountAndSessionIDs(
	ctx context.Context,
	accountID string,
	sessionID string,
) (veriff *model.VeriffSession, err error) {
	veriff = &model.VeriffSession{}

	err = repo.collectionVeriff.FindOne(ctx, bson.M{
		"account_id": accountID,
		"session_id": sessionID,
	}).Decode(veriff)
	if err != nil {
		log.WithError(err).Errorf("SessionGetByAccountAndSessionIDs err: %s", err.Error())
	}

	return
}

func (repo *repositoryImpl) VeriffGetAccountIDByVendorData(
	ctx context.Context,
	vendorData string,
) (veriff *model.VeriffSession, err error) {
	veriff = &model.VeriffSession{}

	err = repo.collectionVeriff.FindOne(ctx, bson.M{
		"vendor_data": vendorData,
	}).Decode(veriff)
	if err != nil {
		log.WithError(err).Errorf("VeriffGetAccountIDByVendorData err: %s", err.Error())
	}

	if err == mongo.ErrNoDocuments {
		veriff = &model.VeriffSession{
			AccountID: "",
		}
		err = nil
	}

	return
}

func (repo *repositoryImpl) CheckApprovedStatusByAccountID(
	ctx context.Context,
	id string,
) (bool, error) {
	adminVeriffs := []model.VeriffSession{}

	adminOpts := options.Find().SetSort(bson.D{{Key: "updated_at", Value: -1}}).SetLimit(1)
	adminFilter := bson.M{
		"$and": bson.A{
			bson.M{"account_id": id},
			bson.M{"admin": true},
		},
	}

	cur, err := repo.collectionVeriff.Find(ctx, adminFilter, adminOpts)
	if err != nil {
		log.WithError(err).Errorf("CheckApprovedStatusByAccountID err: %s", err.Error())
		return false, err
	}

	err = cur.All(ctx, &adminVeriffs)
	if err != nil {
		log.WithError(err).Errorf("CheckApprovedStatusByAccountID err: %s", err.Error())
		return false, err
	}

	if len(adminVeriffs) > 0 {
		for _, v := range adminVeriffs {
			if v.Status == service.VeriffStatusApproved {
				return true, nil
			}
		}

		return false, nil
	}

	veriff := &model.VeriffSession{}

	err = repo.collectionVeriff.FindOne(ctx, bson.M{"$and": bson.A{
		bson.M{"account_id": id},
		bson.M{"$or": bson.A{
			bson.M{"status": service.VeriffStatusApproved},
		}},
	}}).Decode(veriff)
	if err == mongo.ErrNoDocuments {
		return false, nil
	} else if err != nil {
		log.WithError(err).Errorf("CheckApprovedStatusByAccountID err: %s", err.Error())
		return false, err
	}

	return true, nil
}

func (repo *repositoryImpl) VeriffRecordsCountByID(
	ctx context.Context,
	id string,
) (count int64, err error) {
	count, err = repo.collectionVeriff.CountDocuments(ctx, &bson.M{"account_id": id},
		options.Count().SetMaxTime(time.Second*1))
	if err != nil {
		log.WithError(err).Errorf("VeriffRecordsCountByID err: %s", err.Error())
		return -1, err
	}

	return
}

func (repo *repositoryImpl) VeriffListByAccountID(
	ctx context.Context,
	id string,
	params ...*query.QueryParams,
) (veriffs []*model.VeriffSession, err error) {
	cur, err := repo.collectionVeriff.Find(ctx, bson.M{
		"account_id": id,
	})
	if err != nil {
		log.WithError(err).Errorf("VeriffListByAccountID err: %s", err.Error())
		return nil, err
	}

	err = cur.All(ctx, &veriffs)
	if err != nil {
		log.WithError(err).Errorf("VeriffListByAccountID err: %s", err.Error())
		return
	}

	return
}
