// Package service provides implementation for account service
package service

import (
	"fmt"
	"net/http"
	"net/url"

	"gitlab.mashroom.com/mashroom/backend/account/internal/connections"

	"github.com/camunda-cloud/zeebe/clients/go/pkg/worker"
	"github.com/camunda-cloud/zeebe/clients/go/pkg/zbc"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
)

// Wistia external service configuration
type Wistia struct {
	URL               *url.URL
	AccessToken       string
	WebinarsProjectID int32
	HowToProjectID    int32
	HTTPClient        *http.Client
}

// Config for service implementation
type Config struct {
	// Persistent repository
	Repository model.Repository
	// Remote connections
	Connections connections.Connections
	// Wistia config
	Wistia Wistia
	// Zeebe client
	ZeebeClient zbc.Client
}

// Service implementation
type Service interface {
	account.AccountServiceServer
	RegisterProcessHandlers() (worker.JobWorker, error)
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid service config")
	}

	if config.Repository == nil {
		return fmt.Errorf("no repository in service config")
	}

	if config.Connections.Invitation == nil {
		return fmt.Errorf("no invitation connection in service config")
	}

	if config.Connections.Listing == nil {
		return fmt.Errorf("no listing connection in service config")
	}

	if config.Connections.Offer == nil {
		return fmt.Errorf("no offer connection in service config")
	}

	if config.Connections.Notification == nil {
		return fmt.Errorf("no notification connection in service config")
	}

	if config.Connections.Document == nil {
		return fmt.Errorf("no document connection in service config")
	}

	return nil
}

// NewService returns new service instance
func NewService(config *Config) (Service, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	srv := &serviceImpl{
		config: *config,
	}

	return srv, nil
}
