package service

import (
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func (srv *serviceImpl) renderWistiaVideos(ms []*model.WistiaVideo) (videos []*account.WistiaVideo) {
	for _, m := range ms {
		videos = append(videos, srv.renderWistiaVideo(m))
	}

	return
}

func (srv *serviceImpl) renderWistiaVideo(m *model.WistiaVideo) *account.WistiaVideo {
	var typ account.WistiaProjectType

	switch m.Project.ID {
	case srv.config.Wistia.WebinarsProjectID:
		typ = account.WistiaProjectType_WISTIA_PROJECT_TYPE_WEBINARS
	case srv.config.Wistia.HowToProjectID:
		typ = account.WistiaProjectType_WISTIA_PROJECT_TYPE_HOW_TO
	}

	return &account.WistiaVideo{
		Id:          m.ID,
		Name:        m.Name,
		Type:        m.Type,
		Created:     timestamppb.New(m.Created),
		Updated:     timestamppb.New(m.Updated),
		Duration:    m.Duration,
		HashedId:    m.HashedID,
		Description: m.Description,
		Thumbnail: &account.WistiaVideoThumbnail{
			Url:    m.Thumbnail.URL,
			Width:  m.Thumbnail.Width,
			Height: m.Thumbnail.Height,
		},
		Project: &account.WistiaVideoProject{
			Id:       m.Project.ID,
			Name:     m.Project.Name,
			HashedId: m.Project.HashedID,
			Type:     typ,
		},
	}
}

func (srv *serviceImpl) renderAccountLogs(ms []*model.AccountLog) (logs []*account.Log) {
	for _, m := range ms {
		logs = append(logs, srv.renderAccountLog(m))
	}

	return
}

func (srv *serviceImpl) renderAccountLog(m *model.AccountLog) *account.Log {
	return &account.Log{
		Action: string(m.Action),
		Dt:     timestamppb.New(m.Dt),
		User:   m.User,
	}
}

func (srv *serviceImpl) renderAccountPhones(ms []*model.Phone) (phones []*account.Phone) {
	for _, m := range ms {
		phones = append(phones, srv.renderAccountPhone(m))
	}

	return
}

func (srv *serviceImpl) renderAccountPhone(m *model.Phone) *account.Phone {
	return &account.Phone{
		Number: m.Number,
		Type:   m.Type,
	}
}

func (srv *serviceImpl) renderAccounts(ms []*model.Account) (accs []*account.Account) {
	for _, m := range ms {
		accs = append(accs, srv.renderAccount(m))
	}

	return
}

func (srv *serviceImpl) renderVeriffSessions(ms []*model.VeriffSession,
	isAdmin bool,
) (veriffs []*account.VeriffSession) {
	if isAdmin {
		for _, m := range ms {
			veriffs = append(veriffs, srv.renderAdminVeriffSession(m))
		}
	} else {
		for _, m := range ms {
			veriffs = append(veriffs, srv.renderVeriffSession(m))
		}
	}

	return
}

func (srv *serviceImpl) renderVeriffSession(veriff *model.VeriffSession) *account.VeriffSession {
	return &account.VeriffSession{
		DecisionCode: veriff.Code,
		Status:       enumMapping(veriff.Status),
		ReasonCode:   veriff.ReasonCode,
		FirstName:    veriff.FirstName,
		LastName:     veriff.LastName,
		SessionId:    veriff.SessionID,
		AccountId:    veriff.AccountID,
		Url:          veriff.URL,
		Documents:    veriff.Documents,
		Notes:        veriff.Notes,
		IsAdmin:      veriff.IsAdmin,
		CreatedAt:    timestamppb.New(veriff.CreatedAt),
		UpdatedAt:    timestamppb.New(veriff.UpdatedAt),
	}
}

func (srv *serviceImpl) renderAdminVeriffSession(veriff *model.VeriffSession) *account.VeriffSession {
	return &account.VeriffSession{
		DecisionCode: veriff.Code,
		Status:       enumMapping(veriff.Status),
		Reason:       veriff.Reason,
		ReasonCode:   veriff.ReasonCode,
		FirstName:    veriff.FirstName,
		LastName:     veriff.LastName,
		SessionId:    veriff.SessionID,
		AccountId:    veriff.AccountID,
		Url:          veriff.URL,
		Documents:    veriff.Documents,
		Notes:        veriff.Notes,
		IsAdmin:      veriff.IsAdmin,
		CreatedAt:    timestamppb.New(veriff.CreatedAt),
		UpdatedAt:    timestamppb.New(veriff.UpdatedAt),
	}
}

func (srv *serviceImpl) renderOwnerships(
	ov []*model.OwnershipVerification,
) (ownerships []*account.OwnershipVerification) {
	for _, ov := range ov {
		ownerships = append(ownerships, srv.renderOwnership(ov))
	}

	return
}

func (srv *serviceImpl) renderOwnership(ownership *model.OwnershipVerification) *account.OwnershipVerification {
	return &account.OwnershipVerification{
		OwnershipId:  ownership.OwnershipID,
		AccountId:    ownership.AccountID,
		VerifiedById: ownership.VerifiedByID,
		FirstName:    ownership.FirstName,
		LastName:     ownership.LastName,
		Status:       ownershipStatusMapping(ownership.Status),
		IsCompany:    ownership.IsCompany,
		IsPurchased:  ownership.IsPurchased,
		CompanyName:  ownership.CompanyName,
		DocumentIds:  ownership.DocumentIDs,
		CreatedAt:    timestamppb.New(ownership.CreatedAt),
		UpdatedAt:    timestamppb.New(ownership.UpdatedAt),
	}
}

// EnumMapping mapping string to enum
func enumMapping(status string) (value account.VeriffStatus) {
	switch status {
	case VeriffStatusCreated:
		value = account.VeriffStatus_CREATED
	case VeriffStatusUndefined:
		value = account.VeriffStatus_UNDEFINED
	case VeriffStatusApproved:
		value = account.VeriffStatus_APPROVED
	case VeriffStatusResubmissionRequested:
		value = account.VeriffStatus_RESUBMISSION_REQUESTED
	case VeriffStatusDeclined:
		value = account.VeriffStatus_DECLINED
	case VeriffStatusReview:
		value = account.VeriffStatus_REVIEW
	case VeriffStatusSubmitted:
		value = account.VeriffStatus_REVIEW
	case VeriffStatusExpired:
		value = account.VeriffStatus_EXPIRED
	case VeriffStatusAbandoned:
		value = account.VeriffStatus_ABANDONED
	}

	return
}

// GenderMapping mapping gender string to AccountGender
func genderMapping(gender string) account.AccountGender {
	switch gender {
	case "M":
		return account.AccountGender_MALE
	case "F":
		return account.AccountGender_FEMALE
	default:
		return account.AccountGender_UNKNOWN_GENDER
	}
}

// EnumMapping mapping string to enum
func ownershipStatusMapping(status string) (value account.OwnershipStatus) {
	switch status {
	case StatusApproved:
		value = account.OwnershipStatus_OWNERSHIP_APPROVED
	case StatusDeclined:
		value = account.OwnershipStatus_OWNERSHIP_DECLINED
	case StatusPending:
		value = account.OwnershipStatus_OWNERSHIP_PENDING
	default:
		value = account.OwnershipStatus_OWNERSHIP_INITIAL
	}

	return
}

func (srv *serviceImpl) renderAccount(acc *model.Account) *account.Account {
	return &account.Account{
		Id:        &account.Account_IdValue{IdValue: acc.ID},
		CreatedAt: &account.Account_CreatedAtValue{CreatedAtValue: timestamppb.New(acc.CreatedAt)},
		Details: &account.Account_DetailsValue{DetailsValue: &account.AccountDetails{
			FirstName: &account.AccountDetails_FirstNameValue{FirstNameValue: acc.Details.FirstName},
			LastName:  &account.AccountDetails_LastNameValue{LastNameValue: acc.Details.LastName},
			Gender:    &account.AccountDetails_GenderValue{GenderValue: acc.Details.Gender},
			UkNi:      &account.AccountDetails_UkNiValue{UkNiValue: acc.Details.UkNi},
			Birthday:  &account.AccountDetails_BirthdayValue{BirthdayValue: timestamppb.New(acc.Details.Birthday)},
			Avatar:    &account.AccountDetails_AvatarValue{AvatarValue: acc.Details.Avatar},
			FullName:  &account.AccountDetails_FullNameValue{FullNameValue: acc.Details.FullName},
		}},
		Location: &account.Account_LocationValue{LocationValue: &account.AccountLocation{
			Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: acc.Location.Postcode},
			Address1: &account.AccountLocation_Address1Value{Address1Value: acc.Location.Address1},
			Address2: &account.AccountLocation_Address2Value{Address2Value: acc.Location.Address2},
			Address3: &account.AccountLocation_Address3Value{Address3Value: acc.Location.Address3},
			Address4: &account.AccountLocation_Address4Value{Address4Value: acc.Location.Address4},
			Flat:     &account.AccountLocation_FlatValue{FlatValue: acc.Location.Flat},
			Building: &account.AccountLocation_BuildingValue{BuildingValue: acc.Location.Building},
			Street:   &account.AccountLocation_StreetValue{StreetValue: acc.Location.Street},
			City:     &account.AccountLocation_CityValue{CityValue: acc.Location.City},
			Gps: &account.AccountLocation_GpsValue{GpsValue: &account.GpsCoordinates{
				Longitude: acc.Location.GPS.Longitude,
				Latitude:  acc.Location.GPS.Latitude,
			}},
			Country: &account.AccountLocation_CountryValue{CountryValue: acc.Location.Country},
		}},
		Contact: &account.Account_ContactValue{ContactValue: &account.AccountContact{
			EmailPrimary: &account.AccountContact_EmailPrimaryValue{
				EmailPrimaryValue: acc.Contact.EmailPrimary,
			},
			PhonePrimary: &account.AccountContact_PhonePrimaryValue{
				PhonePrimaryValue: acc.Contact.PhonePrimary,
			},
			PhonesSecondary: &account.AccountContact_PhonesSecondaryValue{
				PhonesSecondaryValue: &account.PhoneArrayValue{
					Value: srv.renderAccountPhones(acc.Contact.PhonesSecondary),
				},
			},
			ContactStatus: &account.AccountContact_ContactStatusValue{
				ContactStatusValue: &account.AccountContactStatus{
					EmailVerified: &account.AccountContactStatus_EmailVerifiedValue{
						EmailVerifiedValue: acc.Contact.ContactStatus.EmailVerified,
					},
					PhoneVerified: &account.AccountContactStatus_PhoneVerifiedValue{
						PhoneVerifiedValue: acc.Contact.ContactStatus.PhoneVerified,
					},
				},
			},
		}},
		Favorites: &account.Account_FavoritesValue{FavoritesValue: &account.AccountFavorites{
			Properties: &generic.StringArrayValue{Value: acc.Favorites.Properties},
		}},
		Admin:      &account.Account_AdminValue{AdminValue: acc.Admin},
		Block:      &account.Account_BlockValue{BlockValue: acc.Block},
		TosVersion: &account.Account_TosVersionValue{TosVersionValue: acc.TosVersion},
		LastLogin:  &account.Account_LastLoginValue{LastLoginValue: timestamppb.New(acc.LastLogin)},
		Deleted:    &account.Account_DeletedValue{DeletedValue: acc.Deleted},
		Log:        srv.renderAccountLogs(acc.Log),
		Test:       &account.Account_TestValue{TestValue: acc.Test},
		BankDetails: &account.Account_BankDetailsValue{BankDetailsValue: &account.AccountBankDetails{
			Society:           acc.BankDetails.Society,
			SortCode:          acc.BankDetails.SortCode,
			AccountNumber:     acc.BankDetails.AccountNumber,
			AccountName:       acc.BankDetails.AccountName,
			PersonalReference: acc.BankDetails.PersonalReference,
		}},
		Resources:            nil,
		CreationType:         &account.Account_CreationTypeValue{CreationTypeValue: acc.CreationType},
		ViewedVideosIds:      acc.ViewedVideosIDs,
		EnableLandlordTool:   &account.Account_EnableLandlordToolValue{EnableLandlordToolValue: acc.EnableLandlordTool},
		Entrypoint:           acc.EntryPoint,
		AuthorizationOrigin:  acc.AuthorizationOrigin,
		IdentityVerification: acc.IdentityVerification,
		IsVerified:           acc.IsVerified,
	}
}
