package service

import (
	"bytes"
	"context"
	"errors"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/gocarina/gocsv"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	closecom "gitlab.mashroom.com/mashroom/backend/closecom/api"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcctx"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/bind"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	genericUtil "gitlab.mashroom.com/mashroom/backend/common/generic/util"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	"google.golang.org/protobuf/types/known/timestamppb"
	"google.golang.org/protobuf/types/known/wrapperspb"
)

type serviceImpl struct {
	config Config
}

func (srv *serviceImpl) Verify(
	ctx context.Context,
	req *account.VerifyRequest,
) (resp *account.VerifyResponse, err error) {
	acc := &model.Account{
		CreatedAt: time.Now(),
	}

	if req.GetAccount().GetId() != nil {
		acc, err = srv.config.Repository.AccountGetByID(ctx, req.GetAccount().GetIdValue())
		if err != nil {
			return nil, err
		}
	}

	srv.updateAccount(acc, req.GetAccount(), false)

	valid, verify := srv.validateAccount(acc, req.GetAccount())

	return &account.VerifyResponse{
		Valid:        valid,
		Verification: verify,
	}, nil
}

func (srv *serviceImpl) Create(
	ctx context.Context,
	req *account.CreateRequest,
) (*account.CreateResponse, error) {
	acc := &model.Account{
		ID:                  req.GetAccount().GetIdValue(),
		CreatedAt:           time.Now(),
		Admin:               req.GetAccount().GetAdminValue(),
		CreationType:        req.GetAccount().GetCreationTypeValue(),
		AuthorizationOrigin: req.GetAccount().GetAuthorizationOrigin(),
	}

	srv.updateAccount(acc, req.GetAccount(), false)

	valid, verify := srv.validateAccount(acc, req.GetAccount())

	if !valid {
		return &account.CreateResponse{
			Success:      false,
			Valid:        false,
			Verification: verify,
		}, nil
	}

	err := srv.config.Repository.AccountCreate(ctx, acc)
	if err != nil {
		return nil, err
	}

	err = srv.updateInvitations(ctx, acc)
	if err != nil {
		return nil, err
	}

	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	go srv.serviceAccountLeadExport(bgctx, acc)

	return &account.CreateResponse{
		Account:      srv.renderAccount(acc),
		Success:      true,
		Valid:        valid,
		Verification: verify,
	}, nil
}

func (srv *serviceImpl) Update(
	ctx context.Context,
	req *account.UpdateRequest,
) (*account.UpdateResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetAccount().GetIdValue())
	if err != nil {
		return nil, err
	}

	if acc.Details.FirstName != req.GetAccount().GetDetailsValue().GetFirstNameValue() ||
		acc.Details.LastName != req.GetAccount().GetDetailsValue().GetLastNameValue() {
		isVerified, _ := srv.config.Repository.CheckApprovedStatusByAccountID(ctx, req.GetAccount().GetIdValue())
		if isVerified {
			return nil, errors.New("can't change verified data")
		}
	}

	needUpdateOffers, _ := srv.updateAccount(acc, req.GetAccount(), false)

	valid, verify := srv.validateAccount(acc, req.GetAccount())

	if !valid {
		return &account.UpdateResponse{
			Success:      false,
			Valid:        false,
			Verification: verify,
		}, nil
	}

	if needUpdateOffers {
		err = srv.updateOffers(ctx, acc)
		if err != nil {
			return nil, err
		}
	}

	err = srv.config.Repository.AccountUpdate(ctx, acc)
	if err != nil {
		return nil, err
	}

	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	go srv.updateOffersLL(bgctx, acc)

	go srv.serviceAccountLeadExport(bgctx, acc)

	return &account.UpdateResponse{
		Account:      srv.renderAccount(acc),
		Success:      true,
		Valid:        valid,
		Verification: verify,
	}, nil
}

func (srv *serviceImpl) updateOffersLL(ctx context.Context, acc *model.Account) {
	if acc == nil || acc.ID == "" {
		return
	}

	_, err := srv.config.Connections.Offer.ServiceUpdateOffersLL(
		ctx,
		&offer.ServiceUpdateOffersLLRequest{
			Landlord: &offer.Landlord{
				User:           acc.ID,
				FirstName:      wrapperspb.String(acc.Details.FirstName),
				LastName:       wrapperspb.String(acc.Details.LastName),
				Signature:      nil,
				EmailPrimary:   wrapperspb.String(acc.Contact.EmailPrimary),
				PhonePrimary:   wrapperspb.String(acc.Contact.PhonePrimary),
				BillingAddress: nil,
			},
		},
	)

	if err != nil {
		log.WithError(err).Error("error sending landlord update for active offers")
	}
}

func (srv *serviceImpl) serviceAccountLeadExport(ctx context.Context, m *model.Account) {
	if m.Admin {
		return
	}

	_, err := srv.config.Connections.Closecom.LeadContactSave(ctx, &closecom.LeadContactSaveRequest{
		UserId:     m.ID,
		Name:       genericUtil.JoinNonblankStrings(" ", m.Details.FirstName, m.Details.LastName),
		Email:      m.Contact.EmailPrimary,
		Phone:      m.Contact.PhonePrimary,
		EntryPoint: m.EntryPoint.String(),
		Signup:     timestamppb.New(m.CreatedAt),
	})
	if err != nil {
		log.WithError(err).WithField("user", m.ID).Error("exporting account as lead")
	}
}

func (srv *serviceImpl) UpdateLastLogin(
	ctx context.Context,
	req *account.UpdateLastLoginRequest,
) (*account.UpdateLastLoginResponse, error) {
	err := srv.config.Repository.AccountUpdateLastLogin(ctx, req.GetId())
	if err != nil {
		return nil, err
	}

	return &account.UpdateLastLoginResponse{Success: true}, nil
}

func (srv *serviceImpl) Get(
	ctx context.Context,
	req *account.GetRequest,
) (*account.GetResponse, error) {
	acc, err := srv.config.Repository.AccountGetByIDAdmin(ctx, req.GetId(), req.GetAdmin(), req.GetSoft())
	if err != nil {
		return nil, err
	}

	return &account.GetResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) GetByEmail(
	ctx context.Context,
	req *account.GetByEmailRequest,
) (*account.GetResponse, error) {
	acc, err := srv.config.Repository.AccountGetByEmailAdmin(ctx, req.GetEmail(), req.GetAdmin())
	if err != nil {
		return nil, err
	}

	return &account.GetResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) GetByPhone(
	ctx context.Context,
	req *account.GetByPhoneRequest,
) (*account.GetResponse, error) {
	acc, err := srv.config.Repository.AccountGetByPhoneAdmin(ctx, req.GetPhone(), req.GetAdmin())
	if err != nil {
		return nil, err
	}

	return &account.GetResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) favoriteProperty(ctx context.Context, acc *model.Account, prop string) error {
	for _, p := range acc.Favorites.Properties {
		if p == prop {
			return nil
		}
	}

	_, _ = srv.config.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_PROPERTY_VIEWING_FAVORITE,
		Action: notification.Action_ACTION_CREATE,
		Targets: []*notification.Target{
			{
				User: acc.ID,
				Role: notification.Role_ROLE_TENANT,
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_PROPERTY,
				EmbedValue: prop,
			},
		},
	})

	acc.Favorites.Properties = append(acc.Favorites.Properties, prop)

	return srv.config.Repository.AccountUpdate(ctx, acc)
}

func (srv *serviceImpl) Favorite(
	ctx context.Context,
	req *account.FavoriteRequest,
) (*account.FavoriteResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetUser())
	if err != nil {
		return nil, err
	}

	if req.GetType() == account.AccountFavoriteType_ACCOUNT_FAVORITE_PROPERTY {
		err = srv.favoriteProperty(ctx, acc, req.GetValue())
		if err != nil {
			return nil, err
		}
	}

	return &account.FavoriteResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) unfavoriteProperty(ctx context.Context, acc *model.Account, prop string) error {
	for i, p := range acc.Favorites.Properties {
		if p == prop {
			acc.Favorites.Properties = append(acc.Favorites.Properties[:i], acc.Favorites.Properties[i+1:]...)
			return srv.config.Repository.AccountUpdate(ctx, acc)
		}
	}

	return nil
}

func (srv *serviceImpl) Unfavorite(
	ctx context.Context,
	req *account.UnfavoriteRequest,
) (*account.UnfavoriteResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetUser())
	if err != nil {
		return nil, err
	}

	if req.GetType() == account.AccountFavoriteType_ACCOUNT_FAVORITE_PROPERTY {
		err = srv.unfavoriteProperty(ctx, acc, req.GetValue())
		if err != nil {
			return nil, err
		}
	}

	return &account.UnfavoriteResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) DeleteDocument(
	ctx context.Context,
	req *account.DeleteDocumentRequest,
) (*account.DeleteDocumentResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetUser())
	if err != nil && !req.GetAdminSite() {
		return nil, err
	}

	if err != nil {
		acc, err = srv.config.Repository.AccountGetByAvatar(ctx, req.GetId())
		if err != nil {
			return nil, err
		}
	}

	if acc.Details.Avatar != req.GetId() {
		return nil, errors.New("avatar mismatch")
	}

	acc.Details.Avatar = ""

	err = srv.config.Repository.AccountUpdate(ctx, acc)
	if err != nil {
		return nil, err
	}

	return &account.DeleteDocumentResponse{
		Deleted: true,
		Error:   false,
	}, nil
}

func (srv *serviceImpl) ChangeBankDetails(
	ctx context.Context,
	req *account.ChangeBankDetailsRequest,
) (*account.ChangeBankDetailsResponse, error) {
	details := req.GetBankDetails()

	if details == nil || details.GetAccountNumber() == "" || details.GetSortCode() == "" {
		return &account.ChangeBankDetailsResponse{
			Success: false,
			Errors:  []account.ChangeBankDetailsError{account.ChangeBankDetailsError_BANK_DETAILS_REQUIRED},
		}, nil
	}

	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetUser())
	if err != nil {
		return nil, err
	}

	acc.BankDetails.Society = details.GetSociety()
	acc.BankDetails.SortCode = details.GetSortCode()
	acc.BankDetails.AccountNumber = details.GetAccountNumber()
	acc.BankDetails.AccountName = details.GetAccountName()
	acc.BankDetails.PersonalReference = details.GetPersonalReference()

	if acc.BankDetails.AccountName == "" {
		acc.BankDetails.AccountName = strings.TrimSpace(acc.Details.FirstName + " " + acc.Details.LastName)
	}

	err = srv.config.Repository.AccountUpdate(ctx, acc)
	if err != nil {
		return nil, err
	}

	err = srv.notifyBankDetailsChanged(ctx, acc.ID)
	if err != nil {
		return nil, err
	}

	return &account.ChangeBankDetailsResponse{
		Success: true,
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) ListFilter(
	ctx context.Context,
	req *account.AccountListFilterRequest,
) (*account.ServiceListResponse, error) {
	params := req.GetParamsValue()
	if params == nil {
		params = &query.QueryParams{}
	}

	params.Filter = append(params.Filter, &query.QueryParamsFilter{
		Fields:   []string{"admin"},
		Value:    []string{strconv.FormatBool(req.GetAdmin())},
		Operator: query.QueryFilterOperator_EQUALS,
	})

	accounts, total, err := srv.config.Repository.AccountListByFilter(ctx, params)
	if err != nil {
		return nil, err
	}

	return &account.ServiceListResponse{
		Success:  true,
		Accounts: srv.renderAccounts(accounts),
		Total:    int32(total),
	}, nil
}

func (srv *serviceImpl) ServiceList(
	ctx context.Context,
	req *account.ServiceListRequest,
) (*account.ServiceListResponse, error) {
	accs, _, err := srv.config.Repository.AccountListByFilter(ctx, &query.QueryParams{
		Pagination: &query.QueryParamsPagination{
			Page:    0,
			PerPage: 1000,
		},
		Sort: nil,
		Filter: []*query.QueryParamsFilter{
			{
				Fields:   []string{"admin"},
				Value:    []string{strconv.FormatBool(req.GetAdmin())},
				Operator: query.QueryFilterOperator_EQUALS,
			},
		},
	})
	if err != nil {
		return nil, err
	}

	return &account.ServiceListResponse{Accounts: srv.renderAccounts(accs)}, nil
}

func (srv *serviceImpl) ServiceUpdate(
	ctx context.Context,
	req *account.ServiceUpdateRequest,
) (*account.ServiceUpdateResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetAccount().GetIdValue())
	if err != nil {
		return nil, err
	}

	_, needNotifyBankDetailsChanged := srv.updateAccount(acc, req.GetAccount(), true)

	valid, verify := srv.validateAccount(acc, req.GetAccount())

	if !valid {
		var bs []byte

		bs, err = bind.ToJSON(verify, &bind.Config{GetOmitEmpty: true})
		if err != nil {
			return nil, err
		}

		return nil, grpcerr.New(grpcerr.InvalidData, string(bs))
	}

	err = srv.config.Repository.AccountUpdate(ctx, acc)
	if err != nil {
		return nil, err
	}

	if needNotifyBankDetailsChanged {
		err = srv.notifyBankDetailsChanged(ctx, acc.ID)
		if err != nil {
			return nil, err
		}
	}

	return &account.ServiceUpdateResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) notifyBankDetailsChanged(ctx context.Context, user string) (err error) {
	_, err = srv.config.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_USER_BANKDETAILS,
		Action: notification.Action_ACTION_UPDATE,
		Targets: []*notification.Target{
			{
				User: user,
				Role: notification.Role_ROLE_AUTHOR,
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_USER,
				EmbedValue: user,
			},
		},
	})

	return
}

func (srv *serviceImpl) ServiceDelete(
	ctx context.Context,
	req *account.ServiceDeleteRequest,
) (*account.ServiceDeleteResponse, error) {
	err := srv.config.Repository.AccountDelete(ctx, req.GetId())
	if err != nil {
		return nil, err
	}

	return &account.ServiceDeleteResponse{Success: true}, nil
}

func serviceResourceUpdateList(current, added, removed []string) (res []string) {
	index := make(map[string]bool)

	for _, c := range current {
		index[c] = true
	}

	for _, a := range added {
		index[a] = true
	}

	for _, r := range removed {
		delete(index, r)
	}

	for k := range index {
		res = append(res, k)
	}

	return
}

func (srv *serviceImpl) ServiceResourcesUpdate(
	ctx context.Context,
	req *account.ServiceResourcesUpdateRequest,
) (*account.ServiceResourcesUpdateResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetId())
	if err != nil {
		return nil, err
	}

	acc.Resources.OwnedProperties = serviceResourceUpdateList(
		acc.Resources.OwnedProperties,
		req.GetOwnedProperties().GetAdded(),
		req.GetOwnedProperties().GetRemoved(),
	)

	acc.Resources.ViewingRequests = serviceResourceUpdateList(
		acc.Resources.ViewingRequests,
		req.GetViewingRequests().GetAdded(),
		req.GetViewingRequests().GetRemoved(),
	)

	err = srv.config.Repository.AccountUpdate(ctx, acc)
	if err != nil {
		return nil, err
	}

	return &account.ServiceResourcesUpdateResponse{}, nil
}

func (srv *serviceImpl) Block(
	ctx context.Context,
	req *account.BlockRequest,
) (*account.BlockResponse, error) {
	err := srv.config.Repository.AccountUpdateBlock(ctx, req.GetId(), req.GetBlock())
	if err != nil {
		return nil, err
	}

	return &account.BlockResponse{
		Success: true,
	}, nil
}

func (srv *serviceImpl) DeleteAccount(
	ctx context.Context,
	req *account.DeleteAccountRequest,
) (*account.DeleteAccountResponse, error) {
	err := srv.config.Repository.AccountDeactivate(ctx, req.GetId(), req.GetUser())
	if err != nil {
		return nil, err
	}

	return &account.DeleteAccountResponse{
		Success: true,
	}, nil
}

func (srv *serviceImpl) RestoreAccount(
	ctx context.Context,
	req *account.RestoreAccountRequest,
) (*account.RestoreAccountResponse, error) {
	err := srv.config.Repository.AccountReactivate(ctx, req.GetId(), req.GetUser())
	if err != nil {
		return nil, err
	}

	return &account.RestoreAccountResponse{
		Success: true,
	}, nil
}

func (srv *serviceImpl) AddConfirmation(
	ctx context.Context,
	req *account.ConfirmationRequest,
) (*account.ConfirmationResponse, error) {
	err := srv.config.Repository.AccountUpdateConfirmation(ctx, req.GetUser(), &model.AccountConfirmation{
		Text:      req.GetText(),
		CreatedAt: time.Now(),
		Confirm:   req.GetConfirm(),
	})
	if err != nil {
		return nil, err
	}

	return &account.ConfirmationResponse{
		Success: true,
		Error:   false,
	}, nil
}

type downloadUsersCsvRecord struct {
	FirstName    string `csv:"FirstName"`
	LastName     string `csv:"LastName"`
	EmailAddress string `csv:"EmailAddress"`
}

func (srv *serviceImpl) DownloadUsersCsv(
	ctx context.Context,
	req *account.DownloadUsersCsvRequest,
) (*account.DownloadUsersCsvResponse, error) {
	params := req.GetParamsValue()
	if params == nil {
		params = &query.QueryParams{}
	}

	params.Pagination = nil
	params.Filter = append(params.Filter, &query.QueryParamsFilter{
		Fields:   []string{"admin"},
		Value:    []string{"false"},
		Operator: query.QueryFilterOperator_EQUALS,
	})

	respAcc, _, err := srv.config.Repository.AccountListByFilter(ctx, params)

	if err != nil {
		return nil, err
	}

	var records []*downloadUsersCsvRecord

	for _, acc := range respAcc {
		records = append(records, &downloadUsersCsvRecord{
			FirstName:    acc.Details.FirstName,
			LastName:     acc.Details.LastName,
			EmailAddress: acc.Contact.EmailPrimary,
		})
	}

	bs, err := gocsv.MarshalBytes(records)
	if err != nil {
		return nil, err
	}

	now := time.Now()

	file, err := document.UploadFile(
		ctx,
		srv.config.Connections.Document,
		fmt.Sprintf(
			"userslist-%04d-%02d-%02d-%02d.csv",
			now.Year(),
			now.Month(),
			now.Day(),
			now.Second(),
		),
		"text/csv",
		&document.FileAttribute{
			Data: map[string]string{
				document.FileAttributeGenerated.String(): "true",
			},
		},
		bytes.NewReader(bs),
	)
	if err != nil {
		return nil, err
	}

	return &account.DownloadUsersCsvResponse{
		File:    file.GetId(),
		Success: true,
	}, nil
}
