package service

import (
	"context"
	"errors"
	"time"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic/query"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
)

func (srv *serviceImpl) UpdateOwnershipVerification(
	ctx context.Context, req *account.UpdateOwnershipVerificationRequest,
) (*account.UpdateOwnershipVerificationResponse, error) {
	ownership := &model.OwnershipVerification{
		OwnershipID:  req.GetOwnershipId(),
		AccountID:    req.GetAccountId(),
		VerifiedByID: req.GetVerifiedById(),
		FirstName:    req.GetFirstName(),
		LastName:     req.GetLastName(),
		Status:       account.OwnershipStatus_name[int32(req.GetStatus().Enum().Number())],
		IsCompany:    req.GetIsCompany(),
		IsPurchased:  req.GetIsPurchased(),
		CompanyName:  req.GetCompanyName(),
		Notes:        req.GetNotes(),
		DocumentIDs:  req.GetDocumentIds(),
	}

	if ownership.IsPurchased && len(ownership.DocumentIDs) == 0 {
		return nil, errors.New("documents ids is empty")
	}

	if ownership.IsCompany && ownership.CompanyName == "" {
		return nil, errors.New("company name is empty")
	}

	err := srv.updateOwnershipVerification(ctx, ownership)
	if err != nil {
		log.WithError(err).Errorf("getting updateOwnershipVerification for %s", req.GetAccountId())
		return nil, err
	}

	return &account.UpdateOwnershipVerificationResponse{OwnershipId: ownership.OwnershipID}, nil
}

func (srv *serviceImpl) updateOwnershipVerification(ctx context.Context, v *model.OwnershipVerification) error {
	ownership, err := srv.config.Repository.GetOwnershipVerificationByID(ctx, v.OwnershipID)
	if err != nil {
		log.WithError(err).Errorf("getting GetOwnershipVerificationByID for %s", v.AccountID)
		return err
	}

	srv.updateOwnershipVerificationData(*v, ownership)

	err = srv.config.Repository.UpdateOwnershipVerification(ctx, ownership)
	if err != nil {
		log.WithError(err).Errorf("getting UpdateOwnershipVerification for %s", ownership.AccountID)
		return err
	}

	return nil
}

func (srv *serviceImpl) updateOwnershipVerificationData(
	new model.OwnershipVerification, o *model.OwnershipVerification) {
	if new.FirstName != "" {
		o.FirstName = new.FirstName
	}

	if new.LastName != "" {
		o.LastName = new.LastName
	}

	if new.VerifiedByID != "" {
		o.VerifiedByID = new.VerifiedByID
	}

	o.DocumentIDs = new.DocumentIDs
	o.CompanyName = new.CompanyName
	o.Notes = new.Notes
	o.Status = new.Status
	o.IsCompany = new.IsCompany
	o.IsPurchased = new.IsPurchased
	o.UpdatedAt = time.Now()
}

func (srv *serviceImpl) GetOwnershipVerifications(
	ctx context.Context, req *account.GetOwnershipVerificationRequest,
) (*account.GetOwnershipVerificationResponse, error) {
	params := req.GetParamsValue()
	if params == nil {
		params = &query.QueryParams{}
	}

	if req.PropertyId != nil {
		ownership, err := srv.config.Repository.GetOwnershipVerificationByPropertyID(ctx, req.GetPropertyIdValue())
		if err != nil {
			log.WithError(err).Errorf("getting GetOwnershipVerificationByPropertyID for propertyID: %s",
				req.GetPropertyIdValue())
			return &account.GetOwnershipVerificationResponse{Ownerships: []*account.OwnershipVerification{}}, nil
		}

		return &account.GetOwnershipVerificationResponse{
			Ownerships: []*account.OwnershipVerification{srv.renderOwnership(ownership)},
		}, nil
	}

	if req.OwnershipId != nil {
		ownershipByID, err := srv.config.Repository.GetOwnershipVerificationByID(ctx, req.GetOwnershipIdValue())
		if err != nil {
			log.WithError(err).Errorf("getting GetOwnershipVerificationByID for %s", req.GetOwnershipIdValue())
			return &account.GetOwnershipVerificationResponse{Ownerships: []*account.OwnershipVerification{}}, nil
		}

		return &account.GetOwnershipVerificationResponse{
			Ownerships: []*account.OwnershipVerification{srv.renderOwnership(ownershipByID)},
		}, nil
	}

	if req.AccountId != nil {
		ownerships, accErr := srv.config.Repository.GetOwnershipListByAccountID(ctx, req.GetAccountIdValue(), params)
		if accErr != nil {
			log.WithError(accErr).Errorf("getting GetOwnershipListByAccountID for %s", req.GetAccountIdValue())
			return &account.GetOwnershipVerificationResponse{Ownerships: []*account.OwnershipVerification{}}, accErr
		}

		return &account.GetOwnershipVerificationResponse{Ownerships: srv.renderOwnerships(ownerships)}, nil
	}

	ownershipList, listErr := srv.config.Repository.GetOwnershipList(ctx, params)
	if listErr != nil {
		log.WithError(listErr).Errorf("getting GetOwnershipList")
		return &account.GetOwnershipVerificationResponse{Ownerships: []*account.OwnershipVerification{}}, nil
	}

	return &account.GetOwnershipVerificationResponse{Ownerships: srv.renderOwnerships(ownershipList)}, nil
}
