package service

import (
	"context"

	"github.com/google/uuid"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func (srv *serviceImpl) CreateAdminVerificationSession(
	ctx context.Context, req *account.CreateAdminVerificationSessionRequest,
) (*account.CreateAdminVerificationSessionResponse, error) {
	session := &model.VeriffSession{
		AccountID: req.GetAccountId(),
		SessionID: uuid.New().String(),
		AdminID:   req.GetAdminId(),
		FirstName: req.GetFirstName(),
		LastName:  req.GetLastName(),
		Documents: req.GetDocumentIds(),
		Notes:     req.GetNotes(),
		IsAdmin:   true,
	}

	newSession, err := srv.createAdminVerificationSession(ctx, session, req.GetIsVerified())
	if err != nil {
		log.WithError(err).Errorf("getting createAdminVerificationSession for %s", req.GetAccountId())

		return nil, err
	}

	return newSession, nil
}

func (srv *serviceImpl) createAdminVerificationSession(
	ctx context.Context,
	session *model.VeriffSession,
	isVerified bool,
) (*account.CreateAdminVerificationSessionResponse, error) {
	if isVerified {
		session.Status = VeriffStatusApproved
	} else {
		session.Status = VeriffStatusDeclined
	}

	err := srv.config.Repository.VeriffSessionCreate(ctx, session)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionCreate for %s", session.AccountID)

		return nil, err
	}

	return &account.CreateAdminVerificationSessionResponse{
		Session: &account.VeriffSession{
			Status:    enumMapping(session.Status),
			FirstName: session.FirstName,
			LastName:  session.LastName,
			SessionId: session.SessionID,
			AccountId: session.AccountID,
			Notes:     session.Notes,
			Documents: session.Documents,
			IsAdmin:   session.IsAdmin,
			AdminId:   session.AdminID,
			CreatedAt: timestamppb.New(session.CreatedAt),
		},
	}, nil
}

func (srv *serviceImpl) UpdateAdminVerificationSession(
	ctx context.Context,
	req *account.UpdateAdminVerificationSessionRequest,
) (*account.UpdateAdminVerificationSessionResponse, error) {
	session, err := srv.config.Repository.SessionGetByAccountAndSessionIDs(ctx, req.GetAccountId(), req.GetSessionId())
	if err != nil {
		log.WithError(err).Errorf("getting SessionGetByAccountAndSessionIDs for %s", req.GetAccountId())

		return nil, err
	}

	if !session.IsAdmin {
		log.WithError(err).Errorf("Error update Session without admin status for %s", req.GetAccountId())

		return nil, err
	}

	if req.GetIsVerified() {
		session.Status = VeriffStatusApproved
	} else {
		session.Status = VeriffStatusDeclined
	}

	session.AdminID = req.GetAdminId()
	session.Notes = req.GetNotes()
	session.FirstName = req.GetFirstName()
	session.LastName = req.GetLastName()
	session.Documents = req.GetDocumentIds()

	err = srv.config.Repository.AdminVerifSessionUpdate(ctx, session)
	if err != nil {
		log.WithError(err).Errorf("getting AdminVerifSessionUpdate for %s", req.GetAccountId())

		return nil, err
	}

	return &account.UpdateAdminVerificationSessionResponse{
		Session: &account.VeriffSession{
			Status:    enumMapping(session.Status),
			FirstName: session.FirstName,
			LastName:  session.LastName,
			SessionId: session.SessionID,
			AccountId: session.AccountID,
			Notes:     session.Notes,
			IsAdmin:   session.IsAdmin,
			AdminId:   session.AdminID,
			CreatedAt: timestamppb.New(session.CreatedAt),
		},
	}, nil
}
