package service

import (
	"context"

	"github.com/camunda-cloud/zeebe/clients/go/pkg/entities"
	"github.com/camunda-cloud/zeebe/clients/go/pkg/worker"
	"gitlab.mashroom.com/mashroom/backend/common/generic/bpms"
	"gitlab.mashroom.com/mashroom/backend/common/generic/dbfilter"
)

type contactQuery struct {
	Contact struct {
		EmailPrimary string `json:"email_primary"`
		PhonePrimary string `json:"phone_primary"`
	} `json:"contact"`
	Details struct {
		FirstName string `json:"first_name"`
		LastName  string `json:"last_name"`
	} `json:"details"`
}

func (c *contactQuery) conditions() (conds []dbfilter.Condition) {
	if c == nil {
		return nil
	}

	if c.Contact.EmailPrimary != "" {
		conds = append(conds, dbfilter.Eq("contact.email_primary", c.Contact.EmailPrimary))
	}

	if c.Contact.PhonePrimary != "" {
		conds = append(conds, dbfilter.Eq("contact.phone_primary", c.Contact.PhonePrimary))
	}

	if c.Details.FirstName != "" {
		conds = append(conds, dbfilter.Match("details.first_name", dbfilter.Pattern(c.Details.FirstName)))
	}

	if c.Details.LastName != "" {
		conds = append(conds, dbfilter.Match("details.last_name", dbfilter.Pattern(c.Details.LastName)))
	}

	return
}

func (srv *serviceImpl) ProcessAccountCount() (request interface{}, processor bpms.Processor) {
	req := &contactQuery{}

	return req, func(
		ctx context.Context,
		job entities.Job,
	) (reply interface{}, enhance map[string]interface{}, err error) {
		conds := req.conditions()

		if len(conds) == 0 {
			return map[string]interface{}{"count": 0}, nil, nil
		}

		count, err := srv.config.Repository.AccountCount(ctx, conds...)
		if err != nil {
			return nil, nil, err
		}

		return map[string]interface{}{"count": count}, nil, nil
	}
}

func (srv *serviceImpl) ProcessAccountLookup() (request interface{}, processor bpms.Processor) {
	req := &contactQuery{}

	return req, func(
		ctx context.Context,
		job entities.Job,
	) (reply interface{}, enhance map[string]interface{}, err error) {
		conds := req.conditions()

		if len(conds) == 0 {
			return map[string]interface{}{"exists": false}, nil, nil
		}

		res, err := srv.config.Repository.AccountLookup(ctx, conds...)
		if err != nil {
			return nil, nil, err
		}

		if res == nil {
			return map[string]interface{}{"exists": false}, nil, nil
		}

		return srv.renderAccount(res), map[string]interface{}{"exists": true}, nil
	}
}

// RegisterProcessHandlers sets up BPM process handlers, if BPM is available and returns BPM client, in case it is
func (srv *serviceImpl) RegisterProcessHandlers() (worker.JobWorker, error) {
	if srv.config.ZeebeClient == nil {
		return nil, nil
	}

	handler := bpms.MapHandler(map[string]bpms.Registrator{
		"account-count":  srv.ProcessAccountCount,
		"account-lookup": srv.ProcessAccountLookup,
	})

	return bpms.RegisterProcessHandler(srv.config.ZeebeClient, "account-service", handler)
}
