package service

import (
	"context"
	"encoding/json"
	"errors"
	"time"

	"github.com/google/uuid"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
)

const (
	// StatusInitial ownership was created
	StatusInitial = "OWNERSHIP_INITIAL"

	// StatusPending ownership was created
	StatusPending = "OWNERSHIP_PENDING"

	// StatusApproved valid status
	StatusApproved = "OWNERSHIP_APPROVED"

	// StatusDeclined ownership was declined
	StatusDeclined = "OWNERSHIP_DECLINED"
)

func (srv *serviceImpl) CreateOwnershipVerification(
	ctx context.Context, req *account.CreateOwnershipVerificationRequest,
) (*account.CreateOwnershipVerificationResponse, error) {
	ownership := &model.OwnershipVerification{
		OwnershipID: uuid.New().String(),
		AccountID:   req.GetAccountId(),
		PropertyID:  req.GetPropertyId(),
		FirstName:   req.GetFirstName(),
		LastName:    req.GetLastName(),
		Status:      StatusPending,
		IsCompany:   req.GetIsCompany(),
		IsPurchased: req.GetIsPurchased(),
		CompanyName: req.GetCompanyNameValue(),
		DocumentIDs: req.GetDocumentIds(),
		CreatedAt:   time.Now(),
		UpdatedAt:   time.Now(),
	}

	if ownership.IsPurchased && len(ownership.DocumentIDs) == 0 {
		return nil, errors.New("documents ids is empty")
	}

	if ownership.IsCompany && ownership.CompanyName == "" {
		return nil, errors.New("company name is empty")
	}

	err := srv.createOwnershipVerification(ctx, ownership)
	if err != nil {
		log.WithError(err).Errorf("getting createOwnershipVerification for %s", req.GetAccountId())
		return nil, err
	}

	return &account.CreateOwnershipVerificationResponse{OwnershipId: ownership.OwnershipID}, nil
}

func (srv *serviceImpl) createOwnershipVerification(ctx context.Context, ownership *model.OwnershipVerification) error {
	err := srv.config.Repository.CreateOwnershipVerification(ctx, *ownership)
	if err != nil {
		log.WithError(err).Errorf("getting createOwnershipVerification for %s", ownership.AccountID)
		return err
	}

	srv.sendLandlordNotification(ctx, ownership.AccountID, ownership.OwnershipID)
	srv.sendOwnershipAdminNotification(ctx, ownership)

	return nil
}

func (srv *serviceImpl) OwnershipIsVerified(
	ctx context.Context, req *account.OwnershipIsVerifiedRequest,
) (*account.OwnershipIsVerifiedResponse, error) {
	status, err := srv.config.Repository.CheckOwnershipVerificationByPropertyID(ctx, req.GetPropertyId())
	if err != nil {
		log.WithError(err).Errorf("getting CheckOwnershipVerificationByPropertyID for propertyId%s", req.GetPropertyId())
		return nil, err
	}

	return &account.OwnershipIsVerifiedResponse{Status: ownershipStatusMapping(status)}, nil
}

func (srv *serviceImpl) sendOwnershipAdminNotification(ctx context.Context, ownership *model.OwnershipVerification) {
	bs, _ := json.Marshal(map[string]interface{}{
		"landlord_user": map[string]interface{}{
			"first_name": ownership.FirstName,
			"last_name":  ownership.LastName,
		},
		"property": map[string]interface{}{
			"name": "",
			"id":   ownership.PropertyID,
			"location": map[string]interface{}{
				"postcode": "",
			},
		},
	})

	_, err := srv.config.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_PROPERTY_OWNERSHIP_VERIF,
		Action: notification.Action_ACTION_OWNERSIP_VERIFICATION_CREATE,
		Targets: []*notification.Target{
			{
				Role: notification.Role_ROLE_ADMIN,
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_JSON,
				EmbedValue: string(bs),
			},
		},
	})
	if err != nil {
		log.WithError(err).Errorf("sending sendOwnershipAdminNotification for admin")
	}
}

func (srv *serviceImpl) sendLandlordNotification(ctx context.Context, userID, propertyID string) {
	_, err := srv.config.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_PROPERTY_OWNERSHIP_VERIF,
		Action: notification.Action_ACTION_OWNERSIP_VERIFICATION_CREATE,
		Targets: []*notification.Target{
			{
				User: userID,
				Role: notification.Role_ROLE_LANDLORD,
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_PROPERTY,
				EmbedValue: propertyID,
			},
		},
	})
	if err != nil {
		log.WithError(err).Errorf("sending LandlordNotification for %s", userID)
	}
}
