package service

import (
	"bytes"
	"context"
	"fmt"
	"io/ioutil"
	"net/http"
	"time"

	"gitlab.mashroom.com/mashroom/backend/offer/api/rto"

	"google.golang.org/protobuf/types/known/timestamppb"

	"github.com/google/uuid"

	"gitlab.mashroom.com/mashroom/backend/common/generic/log"

	"gitlab.mashroom.com/mashroom/backend/account/internal/helpers"

	"gitlab.mashroom.com/mashroom/backend/account/internal/model"

	"encoding/json"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
)

const (
	publicKey        string = "7b24aa03-424d-4204-b4be-9573ccd35105"
	privateKey       string = "d73e6c6b-ef2d-4d41-8dff-58d5d8d8f408"
	createSessionURL string = "https://stationapi.veriff.com/v1/sessions/"
	layoutDate       string = "2006-01-02"

	// VeriffStatusApproved valid status
	VeriffStatusApproved = "approved"

	// VeriffStatusUndefined no user's record in db
	VeriffStatusUndefined = "undefined"

	// VeriffStatusNonVerified user don'r verified
	VeriffStatusNonVerified = "non_verified"

	// VeriffStatusCreated when the user's session's was created
	VeriffStatusCreated = "created"

	// VeriffStatusResubmissionRequested needed to be requested
	VeriffStatusResubmissionRequested = "resubmission_requested"

	// VeriffStatusDeclined the user was declined
	VeriffStatusDeclined = "declined"

	// VeriffStatusReview the user is under review
	VeriffStatusReview = "review"

	// VeriffStatusExpired the user's session was expired
	VeriffStatusExpired = "expired"

	// VeriffStatusAbandoned the user is under verification process
	VeriffStatusAbandoned = "abandoned"

	// VeriffStatusSubmitted the user is under review
	VeriffStatusSubmitted = "submitted"
)

func (srv *serviceImpl) CreateVeriffSession(
	ctx context.Context, req *account.CreateVeriffSessionRequest,
) (*account.CreateVeriffSessionResponse, error) {
	session, err := srv.createVeriffSession(ctx, req.GetCallback(), req.GetAccountId())
	if err != nil {
		log.WithError(err).Errorf("getting SessionGetByAccountID for %s", req.GetAccountId())

		return nil, err
	}

	return session, nil
}

func (srv *serviceImpl) UpdateVeriffStatus(
	ctx context.Context,
	req *account.UpdateVeriffStatusRequest,
) (*account.UpdateVeriffStatusResponse, error) {
	v, err := srv.config.Repository.SessionGetByAccountID(ctx, req.GetAccountId())
	if err != nil {
		log.WithError(err).Errorf("getting SessionGetByAccountID for %s", req.GetAccountId())

		return nil, err
	}

	updateResponse, err := updateSessionStatus(v.SessionID, req.GetStatus())
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	v.Status = req.GetStatus()
	v.IsAdmin = false

	err = srv.config.Repository.VeriffSessionUpdate(ctx, v)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionUpdate for %s", req.GetAccountId())

		return nil, err
	}

	return updateResponse, nil
}

func (srv *serviceImpl) DeleteVeriffSession(
	ctx context.Context,
	req *account.DeleteSessionRequest,
) (*account.DeleteSessionResponse, error) {
	v, err := srv.config.Repository.SessionGetByAccountID(ctx, req.GetAccountId())
	if err != nil {
		log.WithError(err).Errorf("getting SessionGetByAccountID for %s", req.GetAccountId())

		return nil, err
	}

	deleteResponse, err := deleteSession(v.SessionID)
	if err != nil {
		log.WithError(err).Error()
		return nil, err
	}

	err = srv.config.Repository.VeriffSessionUpdateByID(ctx, v.SessionID)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionUpdateByID for %s", v.SessionID)

		return nil, err
	}

	return deleteResponse, nil
}

func (srv *serviceImpl) GetVerificationInfo(
	ctx context.Context,
	req *account.GetVerificationRequest,
) (*account.GetVerificationResponse, error) {
	isVerified, err := srv.config.Repository.CheckApprovedStatusByAccountID(ctx, req.GetAccountId())
	if err != nil {
		log.WithError(err).Errorf("getting CheckApprovedStatusByAccountID for %s", req.GetAccountId())

		return nil, err
	}

	veriffs, err := srv.config.Repository.VeriffListByAccountID(ctx, req.GetAccountId())
	if err != nil {
		log.WithError(err).Errorf("getting GetVerificationInfo for %s", req.GetAccountId())

		return nil, err
	}

	return &account.GetVerificationResponse{IsVerified: isVerified,
		Sessions: srv.renderVeriffSessions(veriffs, req.GetIsAdmin()),
	}, nil
}

func (srv *serviceImpl) IsVeriffValid(
	ctx context.Context, req *account.IsVeriffValidRequest,
) (*account.IsVeriffValidResponse, error) {
	isVerified, err := srv.config.Repository.CheckApprovedStatusByAccountID(ctx, req.GetAccountId())
	if err != nil {
		log.WithError(err).Errorf("getting CheckApprovedStatusByAccountID for %s", req.GetAccountId())

		return nil, err
	}

	return &account.IsVeriffValidResponse{Success: isVerified}, nil
}

func updateSessionStatus(sessionID, status string) (*account.UpdateVeriffStatusResponse, error) {
	url := fmt.Sprintf("https://stationapi.veriff.com/v1/sessions/%s", sessionID)
	params := &model.SessionUpdateRequest{
		SessionUpdate: model.SessionUpdate{
			Status:    status,
			Timestamp: time.Now(),
		}}

	body, err := json.Marshal(params)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	signedBody := helpers.ComputeHmac256Bytes(body, privateKey)

	req, err := http.NewRequest(http.MethodPatch, url, bytes.NewBuffer(body))

	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	req.Header.Set("X-AUTH-CLIENT", publicKey)
	req.Header.Set("X-HMAC-SIGNATURE", signedBody)
	req.Header.Add("Content-Type", "application/json; charset=UTF-8")
	res, rErr := http.DefaultClient.Do(req)

	if rErr != nil {
		log.WithError(err).Error()

		return nil, rErr
	}

	data, err := ioutil.ReadAll(res.Body)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	var decision model.VeriffUpdateResponse
	if er := json.Unmarshal(data, &decision); er != nil {
		return nil, er
	}

	defer func() {
		er := res.Body.Close()
		if er != nil {
			log.WithError(err).Error()

			return
		}
	}()

	return &account.UpdateVeriffStatusResponse{
		Id:     decision.VerificationResponse.ID,
		Status: decision.VerificationResponse.Status}, nil
}

func deleteSession(sessionID string) (*account.DeleteSessionResponse, error) {
	url := fmt.Sprintf("https://stationapi.veriff.com/v1/sessions/%s", sessionID)

	signed := helpers.ComputeHmac256(sessionID, privateKey)

	req, err := http.NewRequest(http.MethodDelete, url, nil)

	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	req.Header.Set("X-AUTH-CLIENT", publicKey)
	req.Header.Set("X-HMAC-SIGNATURE", signed)
	req.Header.Add("Content-Type", "application/json; charset=UTF-8")
	res, rErr := http.DefaultClient.Do(req)

	if rErr != nil {
		return nil, rErr
	}

	data, err := ioutil.ReadAll(res.Body)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	fmt.Println("data: ", string(data))

	deleteResponse, err := helpers.DeleteResponseHelper(data)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	defer func() {
		er := res.Body.Close()
		if er != nil {
			log.WithError(err).Error()

			return
		}
	}()

	return deleteResponse, nil
}

func (srv *serviceImpl) EventWebhookResult(
	ctx context.Context,
	req *account.EventWebhookStatusRequest,
) (*account.EventWebhookStatusResponse, error) {
	var event model.EventResponse

	if err := json.Unmarshal(req.GetData(), &event); err != nil {
		log.WithError(err).Errorf("getting %s EventWebhookResult", err.Error())
		return nil, err
	}

	err := srv.config.Repository.VeriffSessionUpdateCodeByVendorData(ctx, event.Code, event.Action, event.VendorData)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionUpdateCodeByVendorData for %s", event.VendorData)
		return nil, err
	}

	return &account.EventWebhookStatusResponse{}, nil
}

func (srv *serviceImpl) DecisionWebhookResult(
	ctx context.Context, req *account.DecisionWebhookStatusRequest,
) (*account.DecisionWebhookStatusResponse, error) {
	var m model.VeriffSession

	var decision model.DecisionResponse

	if err := json.Unmarshal(req.GetData(), &decision); err != nil {
		log.WithError(err).Errorf("getting %s DecisionWebhookResult", err.Error())

		return nil, err
	}

	m.Status = decision.Verification.Status
	m.Reason = decision.Verification.Reason
	m.Code = decision.Verification.Code
	m.ReasonCode = decision.Verification.ReasonCode

	v, err := srv.config.Repository.VeriffGetAccountIDByVendorData(ctx, decision.Verification.VendorData)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionUpdateByVendorData for %s", m.VendorData)

		return nil, err
	}

	if decision.Verification.Status == VeriffStatusApproved {
		acc, accErr := srv.config.Repository.AccountGetByID(ctx, v.AccountID)
		if accErr != nil {
			log.WithError(accErr).Errorf("getting AccountGetByID for %s", v.SessionID)

			return nil, accErr
		}

		acc.IsVerified = true

		srv.updatePersonDataByDecision(&decision, acc)

		err = srv.config.Repository.AccountUpdateByDecision(ctx, v.AccountID, acc)
		if err != nil {
			log.WithError(err).Errorf("getting AccountUpdateByDecision for %s", v.SessionID)

			return nil, err
		}

		go func() {
			_, err = srv.config.Connections.Rto.LandlordApproved(ctx, &rto.LandlordApprovedRequest{
				LandlordId: v.AccountID,
			})
			if err != nil {
				fmt.Printf("Rto.LandlordApproved error: %s\r\n", err.Error())
			}

			fmt.Printf("Rto.LandlordApproved v.AccountID: %s\r\n", v.AccountID)
		}()
	}

	if decision.Verification.Status == VeriffStatusReview {
		fullName := fmt.Sprintf("%s %s", decision.Verification.Person.FirstName, decision.Verification.Person.LastName)

		err = srv.config.Repository.VeriffSessionUpdateFullName(ctx, v.SessionID, fullName)
		if err != nil {
			log.WithError(err).Errorf("getting AccountUpdateByDecision for %s", v.SessionID)
			return nil, err
		}
	}

	err = srv.config.Repository.VeriffSessionUpdateByVendorData(ctx, &m, decision.Verification.VendorData)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionUpdateByVendorData for %s", m.VendorData)

		return nil, err
	}

	return &account.DecisionWebhookStatusResponse{}, nil
}

func (srv *serviceImpl) updatePersonDataByDecision(d *model.DecisionResponse, acc *model.Account) {
	nameChanged := false

	if d.Verification.Person.FirstName != "" {
		acc.Details.FirstName = d.Verification.Person.FirstName
		nameChanged = true
	}

	if d.Verification.Person.LastName != "" {
		acc.Details.LastName = d.Verification.Person.LastName
		nameChanged = true
	}

	if d.Verification.Person.Gender != "" {
		acc.Details.Gender = genderMapping(fmt.Sprintf("%v", d.Verification.Person.Gender))
	}

	if d.Verification.Person.DateOfBirth != "" {
		date, err := time.Parse(layoutDate, d.Verification.Person.DateOfBirth)
		if err != nil {
			acc.Details.Birthday = date
		}
	}

	if nameChanged {
		acc.Details.FullName = fmt.Sprintf("%s %s", acc.Details.FirstName, acc.Details.LastName)
	}
}

func (srv *serviceImpl) createVeriffSession(
	ctx context.Context,
	callbackURL, accountID string,
) (*account.CreateVeriffSessionResponse, error) {
	vendorData := uuid.New().String()
	params := &model.Veriff{
		Verify: model.Verification{
			Callback:   callbackURL,
			VendorData: vendorData,
			Timestamp:  time.Now(),
		},
	}

	resp, err := srv.checkOldVeriffSession(ctx, accountID, params)
	if err != nil {
		return nil, err
	}

	if resp != nil {
		return resp, nil
	}

	result, err := srv.sendDataToVeriff(ctx, params)
	if err != nil {
		return nil, err
	}

	createdAt := time.Now()

	session := &model.VeriffSession{ // for the first creation status will be "created"
		AccountID:  accountID,
		Status:     result.VerificationResponse.Status,
		URL:        result.VerificationResponse.URL,
		VendorData: vendorData,
		SessionID:  result.VerificationResponse.ID,
		CreatedAt:  createdAt,
	}

	acc, err := srv.config.Repository.AccountGetByID(ctx, accountID)
	if err != nil {
		log.WithError(err).Errorf("getting AccountGetByID for %s", accountID)
		return nil, err
	}

	session.FirstName = acc.Details.FirstName
	session.LastName = acc.Details.LastName

	err = srv.config.Repository.VeriffSessionCreate(ctx, session)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionCreate for %s", session.AccountID)

		return nil, err
	}

	return &account.CreateVeriffSessionResponse{
		Session: &account.VeriffSession{
			Status:    enumMapping(result.VerificationResponse.Status),
			FirstName: session.FirstName,
			LastName:  session.LastName,
			SessionId: result.VerificationResponse.ID,
			AccountId: accountID,
			Url:       result.VerificationResponse.URL,
			CreatedAt: timestamppb.New(createdAt),
		},
	}, nil
}

func (srv *serviceImpl) checkOldVeriffSession(
	ctx context.Context,
	accountID string,
	params *model.Veriff,
) (*account.CreateVeriffSessionResponse, error) {
	oldVeriff := srv.getExpiredVeriff(ctx, accountID)
	if oldVeriff == nil {
		return nil, nil
	}

	params.Verify.VendorData = oldVeriff.VendorData
	oldVeriff.Status = VeriffStatusCreated

	result, err := srv.sendDataToVeriff(ctx, params)
	if err != nil {
		return nil, err
	}

	err = srv.config.Repository.VeriffSessionUpdateByVendorData(ctx, oldVeriff, oldVeriff.VendorData)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffSessionUpdateByVendorData for %s", accountID)
	}

	return &account.CreateVeriffSessionResponse{
		Session: &account.VeriffSession{
			Status:    enumMapping(result.VerificationResponse.Status),
			FirstName: oldVeriff.FirstName,
			LastName:  oldVeriff.LastName,
			SessionId: result.VerificationResponse.ID,
			AccountId: accountID,
			Url:       result.VerificationResponse.URL,
			CreatedAt: timestamppb.New(time.Now()),
		},
	}, nil
}

func (srv *serviceImpl) getExpiredVeriff(
	ctx context.Context,
	accountID string,
) *model.VeriffSession {
	veriffs, err := srv.config.Repository.VeriffListByAccountID(ctx, accountID)
	if err != nil {
		log.WithError(err).Errorf("getting VeriffListByAccountID for %s", accountID)
	}

	if veriffs == nil {
		return nil
	}

	for _, v := range veriffs {
		if v.Status == VeriffStatusExpired {
			return v
		}
	}

	return nil
}

func (srv *serviceImpl) sendDataToVeriff(
	ctx context.Context,
	params *model.Veriff,
) (*model.SessionResponse, error) {
	body, err := json.Marshal(params)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	req, err := http.NewRequestWithContext(ctx, http.MethodPost, createSessionURL, bytes.NewBuffer(body))
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	req.Header.Add("X-AUTH-CLIENT", publicKey)
	req.Header.Add("Content-Type", "application/json; charset=UTF-8")

	res, err := http.DefaultClient.Do(req)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	data, err := ioutil.ReadAll(res.Body)
	if err != nil {
		log.WithError(err).Error()

		return nil, err
	}

	defer func() {
		if closeErr := res.Body.Close(); closeErr != nil {
			log.WithError(closeErr).Error()

			return
		}
	}()

	var result model.SessionResponse
	if unmarshalErr := json.Unmarshal(data, &result); unmarshalErr != nil {
		return nil, unmarshalErr
	}

	return &result, nil
}
