package service

import (
	"context"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"strconv"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/api/grpcerr"
)

var errBadHTTPResponseStatus = grpcerr.New(grpcerr.ProviderError, "http response status is not 200")

func (srv *serviceImpl) listWistiaVideos(
	ctx context.Context,
	projectID int32,
) (videos []*model.WistiaVideo, err error) {
	q := srv.config.Wistia.URL.Query()

	q.Set("project_id", strconv.FormatInt(int64(projectID), 10))
	q.Set("type", "Video")

	u := srv.config.Wistia.URL
	u.RawQuery = q.Encode()

	req, err := http.NewRequestWithContext(ctx, http.MethodGet, u.String(), nil)
	if err != nil {
		return nil, err
	}

	req.Header.Set("Authorization", "Bearer "+srv.config.Wistia.AccessToken)

	resp, err := srv.config.Wistia.HTTPClient.Do(req)
	if err != nil {
		return
	}

	defer func() {
		_ = resp.Body.Close()
	}()

	body, err := io.ReadAll(resp.Body)

	if resp.StatusCode != http.StatusOK {
		return nil, fmt.Errorf("%w: %s", errBadHTTPResponseStatus, string(body))
	}

	if err != nil {
		return
	}

	err = json.Unmarshal(body, &videos)
	if err != nil {
		return
	}

	return
}

func (srv *serviceImpl) listSyncWistiaVideos(
	ctx context.Context,
	projectID int32,
) (err error) {
	cur, err := srv.config.Repository.WistiaVideoListByProjectID(ctx, projectID)
	if err != nil {
		return
	}

	index := make(map[int32]bool)

	for _, c := range cur {
		index[c.ID] = true
	}

	resp, err := srv.listWistiaVideos(ctx, projectID)
	if err != nil {
		return err
	}

	for _, r := range resp {
		err = srv.config.Repository.WistiaVideoSave(ctx, r)
		if err != nil {
			return err
		}

		delete(index, r.ID)
	}

	if len(index) == 0 {
		return
	}

	var ids []int32

	for id := range index {
		ids = append(ids, id)
	}

	return srv.config.Repository.WistiaVideoDelete(ctx, ids, projectID)
}

func (srv *serviceImpl) UpdateWistiaCollection(
	ctx context.Context,
	req *account.UpdateWistiaCollectionRequest,
) (*account.UpdateWistiaCollectionResponse, error) {
	err := srv.listSyncWistiaVideos(ctx, srv.config.Wistia.WebinarsProjectID)
	if err != nil {
		return nil, err
	}

	err = srv.listSyncWistiaVideos(ctx, srv.config.Wistia.HowToProjectID)
	if err != nil {
		return nil, err
	}

	return &account.UpdateWistiaCollectionResponse{Success: true}, nil
}

func (srv *serviceImpl) WistiaVideoViewed(
	ctx context.Context,
	req *account.WistiaVideoViewedRequest,
) (*account.WistiaVideoViewedResponse, error) {
	acc, err := srv.config.Repository.AccountGetByID(ctx, req.GetUserId())
	if err != nil {
		return nil, err
	}

	acc.ViewedVideosIDs = append(acc.ViewedVideosIDs, req.GetVideoId())

	err = srv.config.Repository.AccountUpdate(ctx, acc)
	if err != nil {
		return nil, err
	}

	return &account.WistiaVideoViewedResponse{
		Account: srv.renderAccount(acc),
	}, nil
}

func (srv *serviceImpl) ListWistiaVideos(
	ctx context.Context,
	req *account.ListWistiaVideosRequest,
) (*account.ListWistiaVideosResponse, error) {
	var proj int32

	switch req.GetType() {
	case account.WistiaProjectType_WISTIA_PROJECT_TYPE_WEBINARS:
		proj = srv.config.Wistia.WebinarsProjectID
	case account.WistiaProjectType_WISTIA_PROJECT_TYPE_HOW_TO:
		proj = srv.config.Wistia.HowToProjectID
	}

	list, err := srv.config.Repository.WistiaVideoListByProjectID(ctx, proj)
	if err != nil {
		return nil, err
	}

	return &account.ListWistiaVideosResponse{
		WistiaVideos: srv.renderWistiaVideos(list),
	}, nil
}
