package test

import (
	"context"
	"fmt"
	"time"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic"
	"gitlab.mashroom.com/mashroom/backend/common/generic/mockmatcher"
	messaging "gitlab.mashroom.com/mashroom/backend/messaging/api"
	notification "gitlab.mashroom.com/mashroom/backend/notification/api"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func (s *serviceSuite) TestAccountServiceUpdate_ok() {
	const (
		firstName = "foo1"
		lastName  = "foo2"
	)

	id := "000000000000000000000001"
	created := time.Now()

	req := &account.ServiceUpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: firstName,
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: lastName,
					},
					FullName: &account.AccountDetails_FullNameValue{
						FullNameValue: fmt.Sprintf("%s %s",
							firstName,
							lastName,
						),
					},
					Gender: &account.AccountDetails_GenderValue{
						GenderValue: account.AccountGender_MALE,
					},
					UkNi: &account.AccountDetails_UkNiValue{
						UkNiValue: "foo3",
					},
					Birthday: &account.AccountDetails_BirthdayValue{
						BirthdayValue: timestamppb.Now(),
					},
					Avatar: &account.AccountDetails_AvatarValue{
						AvatarValue: "foo4",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: "baz1"},
					Address1: &account.AccountLocation_Address1Value{Address1Value: "baz2"},
					Address2: &account.AccountLocation_Address2Value{Address2Value: "baz3"},
					Address3: &account.AccountLocation_Address3Value{Address3Value: "baz4"},
					Address4: &account.AccountLocation_Address4Value{Address4Value: "baz5"},
					Flat:     &account.AccountLocation_FlatValue{FlatValue: "baz6"},
					Building: &account.AccountLocation_BuildingValue{BuildingValue: "baz7"},
					Street:   &account.AccountLocation_StreetValue{StreetValue: "baz8"},
					City:     &account.AccountLocation_CityValue{CityValue: "baz9"},
					Gps: &account.AccountLocation_GpsValue{GpsValue: &account.GpsCoordinates{
						Longitude: 12,
						Latitude:  34,
					}},
					Country: &account.AccountLocation_CountryValue{CountryValue: "baz10"},
				},
			},
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary: &account.AccountContact_EmailPrimaryValue{
						EmailPrimaryValue: "bar1@example.com",
					},
					PhonePrimary: &account.AccountContact_PhonePrimaryValue{
						PhonePrimaryValue: "123456",
					},
					PhonesSecondary: &account.AccountContact_PhonesSecondaryValue{
						PhonesSecondaryValue: &account.PhoneArrayValue{
							Value: []*account.Phone{
								{
									Number: "123",
									Type:   account.AccountPhoneType_PHONE_TYPE_MOBILE,
								},
							},
						},
					},
					ContactStatus: &account.AccountContact_ContactStatusValue{
						ContactStatusValue: &account.AccountContactStatus{
							EmailVerified: &account.AccountContactStatus_EmailVerifiedValue{
								EmailVerifiedValue: false,
							},
							PhoneVerified: &account.AccountContactStatus_PhoneVerifiedValue{
								PhoneVerifiedValue: false,
							},
						},
					},
				},
			},
			Favorites: &account.Account_FavoritesValue{
				FavoritesValue: &account.AccountFavorites{
					Properties: &generic.StringArrayValue{
						Value: nil,
					},
					Placeholder: false,
				},
			},
			Admin: &account.Account_AdminValue{
				AdminValue: false,
			},
			Block: &account.Account_BlockValue{
				BlockValue: false,
			},
			TosVersion: &account.Account_TosVersionValue{
				TosVersionValue: 0,
			},
			LastLogin: &account.Account_LastLoginValue{
				LastLoginValue: timestamppb.New(time.Time{}),
			},
			Deleted: &account.Account_DeletedValue{
				DeletedValue: false,
			},
			Log: nil,
			Test: &account.Account_TestValue{
				TestValue: false,
			},
			BankDetails: &account.Account_BankDetailsValue{
				BankDetailsValue: &account.AccountBankDetails{
					Society:       "",
					SortCode:      "",
					AccountNumber: "",
					AccountName:   "",
				},
			},
			Resources: nil,
			CreationType: &account.Account_CreationTypeValue{
				CreationTypeValue: 0,
			},
			ViewedVideosIds: nil,
			EnableLandlordTool: &account.Account_EnableLandlordToolValue{
				EnableLandlordToolValue: false,
			},
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	s.repo.EXPECT().AccountUpdate(s.testCtx, mockmatcher.Fields(
		&model.Account{
			ID: id,
			Location: model.AccountLocation{
				Postcode: req.GetAccount().GetLocationValue().GetPostcodeValue(),
				Address1: req.GetAccount().GetLocationValue().GetAddress1Value(),
				Address2: req.GetAccount().GetLocationValue().GetAddress2Value(),
				Address3: req.GetAccount().GetLocationValue().GetAddress3Value(),
				Address4: req.GetAccount().GetLocationValue().GetAddress4Value(),
				Flat:     req.GetAccount().GetLocationValue().GetFlatValue(),
				Building: req.GetAccount().GetLocationValue().GetBuildingValue(),
				Street:   req.GetAccount().GetLocationValue().GetStreetValue(),
				City:     req.GetAccount().GetLocationValue().GetCityValue(),
				GPS: model.GpsCoordinates{
					Longitude: req.GetAccount().GetLocationValue().GetGpsValue().GetLongitude(),
					Latitude:  req.GetAccount().GetLocationValue().GetGpsValue().GetLatitude(),
				},
				Country: req.GetAccount().GetLocationValue().GetCountryValue(),
			},
			Details: model.AccountDetails{
				FirstName: req.GetAccount().GetDetailsValue().GetFirstNameValue(),
				LastName:  req.GetAccount().GetDetailsValue().GetLastNameValue(),
				Gender:    req.GetAccount().GetDetailsValue().GetGenderValue(),
				UkNi:      req.GetAccount().GetDetailsValue().GetUkNiValue(),
				Birthday:  req.GetAccount().GetDetailsValue().GetBirthdayValue().AsTime(),
				Avatar:    req.GetAccount().GetDetailsValue().GetAvatarValue(),
				FullName: fmt.Sprintf("%s %s",
					req.GetAccount().GetDetailsValue().GetFirstNameValue(),
					req.GetAccount().GetDetailsValue().GetLastNameValue(),
				),
			},
			Contact: model.AccountContact{
				EmailPrimary: req.GetAccount().GetContactValue().GetEmailPrimaryValue(),
				PhonePrimary: req.GetAccount().GetContactValue().GetPhonePrimaryValue(),
				PhonesSecondary: []*model.Phone{
					{
						Number: req.GetAccount().GetContactValue().GetPhonesSecondaryValue().GetValue()[0].GetNumber(),
						Type:   req.GetAccount().GetContactValue().GetPhonesSecondaryValue().GetValue()[0].GetType(),
					},
				},
				ContactStatus: model.AccountContactStatus{
					EmailVerified: false,
					PhoneVerified: false,
				},
			},
			Confirmation: model.AccountConfirmation{
				Text:      "",
				CreatedAt: time.Time{},
				Confirm:   false,
			},
			BankDetails: model.AccountBankDetails{
				Society:       "",
				SortCode:      "",
				AccountNumber: "",
				AccountName:   "",
			},
			Resources: model.AccountResources{
				OwnedProperties: nil,
				ViewingRequests: nil,
			},
			Favorites: model.AccountFavorites{
				Properties: nil,
			},
			Log:                 nil,
			ViewedVideosIDs:     nil,
			CreatedAt:           time.Time{},
			LastLogin:           time.Time{},
			TosVersion:          0,
			CreationType:        0,
			Admin:               false,
			Block:               false,
			Deleted:             false,
			Test:                false,
			EnableLandlordTool:  false,
			EntryPoint:          0,
			AuthorizationOrigin: 0,
		},
		"CreatedAt",
	)).Do(func(ctx context.Context, v *model.Account) {
		v.ID = id
		v.CreatedAt = created
		req.Account.Id = &account.Account_IdValue{IdValue: id}
		req.Account.CreatedAt = &account.Account_CreatedAtValue{CreatedAtValue: timestamppb.New(created)}
	})

	s.notification.EXPECT().SendNotification(s.testCtx, &notification.SendNotificationRequest{
		Entity: notification.Entity_ENTITY_USER_BANKDETAILS,
		Action: notification.Action_ACTION_UPDATE,
		Targets: []*notification.Target{
			{
				User: id,
				Role: notification.Role_ROLE_AUTHOR,
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_USER,
				EmbedValue: id,
			},
		},
	})

	resp, err := s.srv.ServiceUpdate(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().EqualValues(req.GetAccount(), resp.GetAccount())
}

func (s *serviceSuite) TestAccountServiceUpdate_error_details() {
	id := "000000000000000000000001"

	req := &account.ServiceUpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{},
			},
		},
	}

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	resp, err := s.srv.ServiceUpdate(s.testCtx, req)

	s.Require().Error(err)

	s.Require().Nil(resp)
}

func (s *serviceSuite) TestAccountServiceUpdate_error_location() {
	id := "000000000000000000000001"

	req := &account.ServiceUpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: "foo1",
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: "foo2",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: ""},
				},
			},
		},
	}

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	resp, err := s.srv.ServiceUpdate(s.testCtx, req)

	s.Require().Error(err)

	s.Require().Nil(resp)
}
