package test

import (
	"context"
	"fmt"
	"sync"
	"time"

	"github.com/golang/mock/gomock"
	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	closecom "gitlab.mashroom.com/mashroom/backend/closecom/api"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic"
	"gitlab.mashroom.com/mashroom/backend/common/generic/mockmatcher"
	genericUtil "gitlab.mashroom.com/mashroom/backend/common/generic/util"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	"google.golang.org/grpc"
	"google.golang.org/protobuf/types/known/timestamppb"
	"google.golang.org/protobuf/types/known/wrapperspb"
)

func (s *serviceSuite) TestAccountUpdate_ok() {
	const (
		firstName = "foo1"
		lastName  = "foo2"
	)

	id := "000000000000000000000001"
	created := time.Now()

	req := &account.UpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: firstName,
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: lastName,
					},
					FullName: &account.AccountDetails_FullNameValue{
						FullNameValue: fmt.Sprintf("%s %s",
							firstName,
							lastName,
						),
					},
					Gender: &account.AccountDetails_GenderValue{
						GenderValue: account.AccountGender_MALE,
					},
					UkNi: &account.AccountDetails_UkNiValue{
						UkNiValue: "foo3",
					},
					Birthday: &account.AccountDetails_BirthdayValue{
						BirthdayValue: timestamppb.Now(),
					},
					Avatar: &account.AccountDetails_AvatarValue{
						AvatarValue: "foo4",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: "baz1"},
					Address1: &account.AccountLocation_Address1Value{Address1Value: "baz2"},
					Address2: &account.AccountLocation_Address2Value{Address2Value: "baz3"},
					Address3: &account.AccountLocation_Address3Value{Address3Value: "baz4"},
					Address4: &account.AccountLocation_Address4Value{Address4Value: "baz5"},
					Flat:     &account.AccountLocation_FlatValue{FlatValue: "baz6"},
					Building: &account.AccountLocation_BuildingValue{BuildingValue: "baz7"},
					Street:   &account.AccountLocation_StreetValue{StreetValue: "baz8"},
					City:     &account.AccountLocation_CityValue{CityValue: "baz9"},
					Gps: &account.AccountLocation_GpsValue{GpsValue: &account.GpsCoordinates{
						Longitude: 12,
						Latitude:  34,
					}},
					Country: &account.AccountLocation_CountryValue{CountryValue: "baz10"},
				},
			},
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary: &account.AccountContact_EmailPrimaryValue{
						EmailPrimaryValue: "bar1@example.com",
					},
					PhonePrimary: &account.AccountContact_PhonePrimaryValue{
						PhonePrimaryValue: "123456",
					},
					PhonesSecondary: &account.AccountContact_PhonesSecondaryValue{
						PhonesSecondaryValue: &account.PhoneArrayValue{
							Value: []*account.Phone{
								{
									Number: "123",
									Type:   account.AccountPhoneType_PHONE_TYPE_MOBILE,
								},
							},
						},
					},
					ContactStatus: &account.AccountContact_ContactStatusValue{
						ContactStatusValue: &account.AccountContactStatus{
							EmailVerified: &account.AccountContactStatus_EmailVerifiedValue{
								EmailVerifiedValue: false,
							},
							PhoneVerified: &account.AccountContactStatus_PhoneVerifiedValue{
								PhoneVerifiedValue: false,
							},
						},
					},
				},
			},
			Favorites: &account.Account_FavoritesValue{
				FavoritesValue: &account.AccountFavorites{
					Properties: &generic.StringArrayValue{
						Value: nil,
					},
					Placeholder: false,
				},
			},
			Admin: &account.Account_AdminValue{
				AdminValue: false,
			},
			Block: &account.Account_BlockValue{
				BlockValue: false,
			},
			TosVersion: &account.Account_TosVersionValue{
				TosVersionValue: 0,
			},
			LastLogin: &account.Account_LastLoginValue{
				LastLoginValue: timestamppb.New(time.Time{}),
			},
			Deleted: &account.Account_DeletedValue{
				DeletedValue: false,
			},
			Log: nil,
			Test: &account.Account_TestValue{
				TestValue: false,
			},
			BankDetails: &account.Account_BankDetailsValue{
				BankDetailsValue: &account.AccountBankDetails{
					Society:       "",
					SortCode:      "",
					AccountNumber: "",
					AccountName:   "",
				},
			},
			Resources: nil,
			CreationType: &account.Account_CreationTypeValue{
				CreationTypeValue: 0,
			},
			ViewedVideosIds: nil,
			EnableLandlordTool: &account.Account_EnableLandlordToolValue{
				EnableLandlordToolValue: false,
			},
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}

	s.repo.EXPECT().CheckApprovedStatusByAccountID(s.testCtx, id).Return(false, nil)

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	s.offer.EXPECT().ListIncoming(s.testCtx, gomock.Any()).Return(&offer.ListResponse{
		Offers: []*offer.Offer{
			{
				Id: "id1",
			},
		},
		Errors: nil,
	}, nil)

	s.offer.EXPECT().ServiceUpdate(s.testCtx, &offer.ServiceUpdateRequest{
		Offer: &offer.Offer{
			Id: "id1",
			Landlord: &offer.Landlord{
				BillingAddress: wrapperspb.String(genericUtil.JoinNonblankStrings(
					" ",
					req.GetAccount().GetLocationValue().GetAddress1Value(),
					req.GetAccount().GetLocationValue().GetAddress2Value(),
					req.GetAccount().GetLocationValue().GetCityValue(),
					req.GetAccount().GetLocationValue().GetCountryValue(),
					req.GetAccount().GetLocationValue().GetPostcodeValue(),
				)),
				FirstName:    wrapperspb.String(req.GetAccount().GetDetailsValue().GetFirstNameValue()),
				LastName:     wrapperspb.String(req.GetAccount().GetDetailsValue().GetLastNameValue()),
				EmailPrimary: wrapperspb.String(req.GetAccount().GetContactValue().GetEmailPrimaryValue()),
				PhonePrimary: wrapperspb.String(req.GetAccount().GetContactValue().GetPhonePrimaryValue()),
			},
		},
	})

	s.offer.EXPECT().ListOutgoing(s.testCtx, gomock.Any()).Return(&offer.ListResponse{
		Offers: []*offer.Offer{
			{
				Id: "id2",
				Tenants: &offer.TenantArrayValue{
					Value: []*offer.Tenant{
						{
							User: id,
						},
					},
				},
			},
		},
		Errors: nil,
	}, nil)

	s.offer.EXPECT().ServiceUpdate(s.testCtx, &offer.ServiceUpdateRequest{
		Offer: &offer.Offer{
			Id: "id2",
			Tenants: &offer.TenantArrayValue{
				Value: []*offer.Tenant{
					{
						User: id,
						BillingAddress: wrapperspb.String(genericUtil.JoinNonblankStrings(
							" ",
							req.GetAccount().GetLocationValue().GetAddress1Value(),
							req.GetAccount().GetLocationValue().GetAddress2Value(),
							req.GetAccount().GetLocationValue().GetCityValue(),
							req.GetAccount().GetLocationValue().GetCountryValue(),
							req.GetAccount().GetLocationValue().GetPostcodeValue(),
						)),
						FirstName:    wrapperspb.String(req.GetAccount().GetDetailsValue().GetFirstNameValue()),
						LastName:     wrapperspb.String(req.GetAccount().GetDetailsValue().GetLastNameValue()),
						EmailPrimary: wrapperspb.String(req.GetAccount().GetContactValue().GetEmailPrimaryValue()),
						PhonePrimary: wrapperspb.String(req.GetAccount().GetContactValue().GetPhonePrimaryValue()),
					},
				},
			},
		},
	})

	s.repo.EXPECT().AccountUpdate(s.testCtx, mockmatcher.Fields(
		&model.Account{
			ID: id,
			Location: model.AccountLocation{
				Postcode: req.GetAccount().GetLocationValue().GetPostcodeValue(),
				Address1: req.GetAccount().GetLocationValue().GetAddress1Value(),
				Address2: req.GetAccount().GetLocationValue().GetAddress2Value(),
				Address3: req.GetAccount().GetLocationValue().GetAddress3Value(),
				Address4: req.GetAccount().GetLocationValue().GetAddress4Value(),
				Flat:     req.GetAccount().GetLocationValue().GetFlatValue(),
				Building: req.GetAccount().GetLocationValue().GetBuildingValue(),
				Street:   req.GetAccount().GetLocationValue().GetStreetValue(),
				City:     req.GetAccount().GetLocationValue().GetCityValue(),
				GPS: model.GpsCoordinates{
					Longitude: req.GetAccount().GetLocationValue().GetGpsValue().GetLongitude(),
					Latitude:  req.GetAccount().GetLocationValue().GetGpsValue().GetLatitude(),
				},
				Country: req.GetAccount().GetLocationValue().GetCountryValue(),
			},
			Details: model.AccountDetails{
				FirstName: req.GetAccount().GetDetailsValue().GetFirstNameValue(),
				LastName:  req.GetAccount().GetDetailsValue().GetLastNameValue(),
				Gender:    req.GetAccount().GetDetailsValue().GetGenderValue(),
				UkNi:      req.GetAccount().GetDetailsValue().GetUkNiValue(),
				Birthday:  req.GetAccount().GetDetailsValue().GetBirthdayValue().AsTime(),
				Avatar:    req.GetAccount().GetDetailsValue().GetAvatarValue(),
				FullName: fmt.Sprintf("%s %s",
					req.GetAccount().GetDetailsValue().GetFirstNameValue(),
					req.GetAccount().GetDetailsValue().GetLastNameValue(),
				),
			},
			Contact: model.AccountContact{
				EmailPrimary: req.GetAccount().GetContactValue().GetEmailPrimaryValue(),
				PhonePrimary: req.GetAccount().GetContactValue().GetPhonePrimaryValue(),
				PhonesSecondary: []*model.Phone{
					{
						Number: req.GetAccount().GetContactValue().GetPhonesSecondaryValue().GetValue()[0].GetNumber(),
						Type:   req.GetAccount().GetContactValue().GetPhonesSecondaryValue().GetValue()[0].GetType(),
					},
				},
				ContactStatus: model.AccountContactStatus{
					EmailVerified: false,
					PhoneVerified: false,
				},
			},
			Confirmation: model.AccountConfirmation{
				Text:      "",
				CreatedAt: time.Time{},
				Confirm:   false,
			},
			BankDetails: model.AccountBankDetails{
				Society:       "",
				SortCode:      "",
				AccountNumber: "",
				AccountName:   "",
			},
			Resources: model.AccountResources{
				OwnedProperties: nil,
				ViewingRequests: nil,
			},
			Favorites: model.AccountFavorites{
				Properties: nil,
			},
			Log:                 nil,
			ViewedVideosIDs:     nil,
			CreatedAt:           time.Time{},
			LastLogin:           time.Time{},
			TosVersion:          0,
			CreationType:        0,
			Admin:               false,
			Block:               false,
			Deleted:             false,
			Test:                false,
			EnableLandlordTool:  false,
			EntryPoint:          0,
			AuthorizationOrigin: 0,
		},
		"CreatedAt",
	)).Do(func(ctx context.Context, v *model.Account) {
		v.ID = id
		v.CreatedAt = created
		req.Account.Id = &account.Account_IdValue{IdValue: id}
		req.Account.CreatedAt = &account.Account_CreatedAtValue{CreatedAtValue: timestamppb.New(created)}
	})

	wg := &sync.WaitGroup{}
	wg.Add(2)

	s.offer.EXPECT().ServiceUpdateOffersLL(
		gomock.Any(),
		&offer.ServiceUpdateOffersLLRequest{
			Landlord: &offer.Landlord{
				User:           id,
				FirstName:      wrapperspb.String(req.GetAccount().GetDetailsValue().GetFirstNameValue()),
				LastName:       wrapperspb.String(req.GetAccount().GetDetailsValue().GetLastNameValue()),
				Signature:      nil,
				EmailPrimary:   wrapperspb.String(req.GetAccount().GetContactValue().GetEmailPrimaryValue()),
				PhonePrimary:   wrapperspb.String(req.GetAccount().GetContactValue().GetPhonePrimaryValue()),
				BillingAddress: nil,
			},
		},
	).Do(func(c context.Context, r *offer.ServiceUpdateOffersLLRequest, opts ...grpc.CallOption) {
		wg.Done()
	})

	s.closecom.EXPECT().LeadContactSave(gomock.Any(), &closecom.LeadContactSaveRequest{
		UserId: id,
		Name: genericUtil.JoinNonblankStrings(
			" ",
			req.GetAccount().GetDetailsValue().GetFirstNameValue(),
			req.GetAccount().GetDetailsValue().GetLastNameValue(),
		),
		Email:      req.GetAccount().GetContactValue().GetEmailPrimaryValue(),
		Phone:      req.GetAccount().GetContactValue().GetPhonePrimaryValue(),
		EntryPoint: req.GetAccount().GetEntrypoint().String(),
		Signup:     timestamppb.New(created),
	}).Do(func(c context.Context, r *closecom.LeadContactSaveRequest, opts ...grpc.CallOption) {
		wg.Done()
	})

	resp, err := s.srv.Update(s.testCtx, req)

	wg.Wait()

	s.Require().NoError(err)

	s.Require().True(resp.GetValid())

	s.Require().EqualValues(req.GetAccount(), resp.GetAccount())
}

func (s *serviceSuite) TestAccountUpdate_error_details() {
	id := "000000000000000000000001"

	req := &account.UpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{},
			},
		},
	}

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	resp, err := s.srv.Update(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().False(resp.GetValid())

	s.Require().Len(resp.GetVerification().GetDetails().GetFirstName().GetErrors(), 1)
	s.Require().Len(resp.GetVerification().GetDetails().GetLastName().GetErrors(), 1)
}

func (s *serviceSuite) TestAccountUpdate_error_location() {
	id := "000000000000000000000001"

	req := &account.UpdateRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: "foo1",
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: "foo2",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: ""},
				},
			},
		},
	}

	s.repo.EXPECT().CheckApprovedStatusByAccountID(s.testCtx, id).Return(false, nil)

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	resp, err := s.srv.Update(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().False(resp.GetValid())

	s.Require().Len(resp.GetVerification().GetLocation().GetPostcode().GetErrors(), 1)
}
