package test

import (
	"time"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/common/api/generic"
	"google.golang.org/protobuf/types/known/timestamppb"
)

func (s *serviceSuite) TestAccountVerify_ok() {
	id := "000000000000000000000001"

	req := &account.VerifyRequest{
		Account: &account.Account{
			Id: &account.Account_IdValue{
				IdValue: id,
			},
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: "foo1",
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: "foo2",
					},
					Gender: &account.AccountDetails_GenderValue{
						GenderValue: account.AccountGender_MALE,
					},
					UkNi: &account.AccountDetails_UkNiValue{
						UkNiValue: "foo3",
					},
					Birthday: &account.AccountDetails_BirthdayValue{
						BirthdayValue: timestamppb.Now(),
					},
					Avatar: &account.AccountDetails_AvatarValue{
						AvatarValue: "foo4",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: "baz1"},
					Address1: &account.AccountLocation_Address1Value{Address1Value: "baz2"},
					Address2: &account.AccountLocation_Address2Value{Address2Value: "baz3"},
					Address3: &account.AccountLocation_Address3Value{Address3Value: "baz4"},
					Address4: &account.AccountLocation_Address4Value{Address4Value: "baz5"},
					Flat:     &account.AccountLocation_FlatValue{FlatValue: "baz6"},
					Building: &account.AccountLocation_BuildingValue{BuildingValue: "baz7"},
					Street:   &account.AccountLocation_StreetValue{StreetValue: "baz8"},
					City:     &account.AccountLocation_CityValue{CityValue: "baz9"},
					Gps: &account.AccountLocation_GpsValue{GpsValue: &account.GpsCoordinates{
						Longitude: 12,
						Latitude:  34,
					}},
					Country: &account.AccountLocation_CountryValue{CountryValue: "baz10"},
				},
			},
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary: &account.AccountContact_EmailPrimaryValue{
						EmailPrimaryValue: "bar1@example.com",
					},
					PhonePrimary: &account.AccountContact_PhonePrimaryValue{
						PhonePrimaryValue: "123456",
					},
					PhonesSecondary: &account.AccountContact_PhonesSecondaryValue{
						PhonesSecondaryValue: &account.PhoneArrayValue{
							Value: []*account.Phone{
								{
									Number: "123",
									Type:   account.AccountPhoneType_PHONE_TYPE_MOBILE,
								},
							},
						},
					},
					ContactStatus: &account.AccountContact_ContactStatusValue{
						ContactStatusValue: &account.AccountContactStatus{
							EmailVerified: &account.AccountContactStatus_EmailVerifiedValue{
								EmailVerifiedValue: false,
							},
							PhoneVerified: &account.AccountContactStatus_PhoneVerifiedValue{
								PhoneVerifiedValue: false,
							},
						},
					},
				},
			},
			Favorites: &account.Account_FavoritesValue{
				FavoritesValue: &account.AccountFavorites{
					Properties: &generic.StringArrayValue{
						Value: nil,
					},
					Placeholder: false,
				},
			},
			Admin: &account.Account_AdminValue{
				AdminValue: false,
			},
			Block: &account.Account_BlockValue{
				BlockValue: false,
			},
			TosVersion: &account.Account_TosVersionValue{
				TosVersionValue: 0,
			},
			LastLogin: &account.Account_LastLoginValue{
				LastLoginValue: timestamppb.New(time.Time{}),
			},
			Deleted: &account.Account_DeletedValue{
				DeletedValue: false,
			},
			Log: nil,
			Test: &account.Account_TestValue{
				TestValue: false,
			},
			BankDetails: &account.Account_BankDetailsValue{
				BankDetailsValue: &account.AccountBankDetails{
					Society:       "",
					SortCode:      "",
					AccountNumber: "",
					AccountName:   "",
				},
			},
			Resources: nil,
			CreationType: &account.Account_CreationTypeValue{
				CreationTypeValue: 0,
			},
			ViewedVideosIds: nil,
			EnableLandlordTool: &account.Account_EnableLandlordToolValue{
				EnableLandlordToolValue: false,
			},
			Entrypoint:          0,
			AuthorizationOrigin: 0,
		},
	}

	s.repo.EXPECT().AccountGetByID(s.testCtx, id).Return(&model.Account{
		ID: id,
	}, nil)

	resp, err := s.srv.Verify(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().True(resp.GetValid())
}

func (s *serviceSuite) TestAccountVerify_error_details() {
	req := &account.CreateRequest{
		Account: &account.Account{
			Id:        nil,
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{},
			},
		},
	}

	resp, err := s.srv.Create(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().False(resp.GetValid())

	s.Require().Len(resp.GetVerification().GetDetails().GetFirstName().GetErrors(), 1)
	s.Require().Len(resp.GetVerification().GetDetails().GetLastName().GetErrors(), 1)
}

func (s *serviceSuite) TestAccountVerify_error_location() {
	req := &account.VerifyRequest{
		Account: &account.Account{
			Id:        nil,
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: "foo1",
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: "foo2",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: ""},
				},
			},
		},
	}

	resp, err := s.srv.Verify(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().False(resp.GetValid())

	s.Require().Len(resp.GetVerification().GetLocation().GetPostcode().GetErrors(), 1)
}

func (s *serviceSuite) TestAccountVerify_error_contact() {
	req := &account.VerifyRequest{
		Account: &account.Account{
			Id:        nil,
			CreatedAt: nil,
			Details: &account.Account_DetailsValue{
				DetailsValue: &account.AccountDetails{
					FirstName: &account.AccountDetails_FirstNameValue{
						FirstNameValue: "foo1",
					},
					LastName: &account.AccountDetails_LastNameValue{
						LastNameValue: "foo2",
					},
				},
			},
			Location: &account.Account_LocationValue{
				LocationValue: &account.AccountLocation{
					Postcode: &account.AccountLocation_PostcodeValue{PostcodeValue: "abc"},
				},
			},
			Contact: &account.Account_ContactValue{
				ContactValue: &account.AccountContact{
					EmailPrimary: &account.AccountContact_EmailPrimaryValue{
						EmailPrimaryValue: "",
					},
					PhonePrimary: &account.AccountContact_PhonePrimaryValue{
						PhonePrimaryValue: "",
					},
					PhonesSecondary: &account.AccountContact_PhonesSecondaryValue{
						PhonesSecondaryValue: &account.PhoneArrayValue{
							Value: []*account.Phone{
								{
									Number: "",
									Type:   account.AccountPhoneType_PHONE_TYPE_MOBILE,
								},
							},
						},
					},
				},
			},
		},
	}

	resp, err := s.srv.Verify(s.testCtx, req)

	s.Require().NoError(err)

	s.Require().False(resp.GetValid())

	s.Require().Len(resp.GetVerification().GetContact().GetEmailPrimary().GetErrors(), 1)
}
