package test

import (
	"context"
	"encoding/json"
	"net/http"
	"net/http/httptest"
	"net/url"
	"strconv"
	"testing"

	"github.com/golang/mock/gomock"
	"github.com/stretchr/testify/suite"
	"gitlab.mashroom.com/mashroom/backend/account/internal/connections"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	repo "gitlab.mashroom.com/mashroom/backend/account/internal/model/mock"
	"gitlab.mashroom.com/mashroom/backend/account/internal/service"
	closecom "gitlab.mashroom.com/mashroom/backend/closecom/api"
	mockclosecom "gitlab.mashroom.com/mashroom/backend/closecom/api/mock"
	mockdocument "gitlab.mashroom.com/mashroom/backend/document/api/mock"
	mockinvitation "gitlab.mashroom.com/mashroom/backend/invitation/api/mock"
	mocklisting "gitlab.mashroom.com/mashroom/backend/listing/api/mock"
	mocknotification "gitlab.mashroom.com/mashroom/backend/notification/api/mock"
	mockoffer "gitlab.mashroom.com/mashroom/backend/offer/api/mock"
	mockviewing "gitlab.mashroom.com/mashroom/backend/viewing/api/mock"
	mockzeebe "gitlab.mashroom.com/mashroom/backend/zeebe/api/mock"
)

type contextWistiaVideosT struct {
}

var contextWistiaVideosKey = contextWistiaVideosT{}

type serviceSuite struct {
	suite.Suite
	srv                    service.Service
	rootCtx, testCtx       context.Context
	rootCancel, testCancel context.CancelFunc
	ctrl                   *gomock.Controller
	repo                   *repo.MockRepository
	invitation             *mockinvitation.MockInvitationServiceClient
	listing                *mocklisting.MockListingServiceClient
	offer                  *mockoffer.MockOfferServiceClient
	notification           *mocknotification.MockNotificationServiceClient
	document               *mockdocument.MockDocumentServiceClient
	closecom               *mockclosecom.MockClosecomServiceClient
	viewing                *mockviewing.MockViewingServiceClient
	zbcclient              *mockzeebe.MockClient
}

func (s *serviceSuite) SetupSuite() {
	s.rootCtx, s.rootCancel = context.WithCancel(context.Background())
}

func (s *serviceSuite) SetupTest() {
	s.ctrl = gomock.NewController(s.T())
	s.repo = repo.NewMockRepository(s.ctrl)
	s.invitation = mockinvitation.NewMockInvitationServiceClient(s.ctrl)
	s.listing = mocklisting.NewMockListingServiceClient(s.ctrl)
	s.offer = mockoffer.NewMockOfferServiceClient(s.ctrl)
	s.notification = mocknotification.NewMockNotificationServiceClient(s.ctrl)
	s.document = mockdocument.NewMockDocumentServiceClient(s.ctrl)
	s.closecom = mockclosecom.NewMockClosecomServiceClient(s.ctrl)
	s.viewing = mockviewing.NewMockViewingServiceClient(s.ctrl)
	s.zbcclient = mockzeebe.NewMockClient(s.ctrl)
	s.testCtx, s.testCancel = context.WithCancel(s.rootCtx)

	var err error

	var videos []*model.WistiaVideo

	s.testCtx = context.WithValue(s.testCtx, contextWistiaVideosKey, &videos)

	wistiasrv := httptest.NewServer(http.HandlerFunc(func(writer http.ResponseWriter, request *http.Request) {
		bs, _ := json.Marshal(videos)

		writer.Header().Set("Content-Length", strconv.Itoa(len(bs)))
		writer.WriteHeader(http.StatusOK)

		_, _ = writer.Write(bs)
	}))

	wistiaurl, err := url.Parse(wistiasrv.URL)
	s.Require().NoError(err)

	wistiaclient := wistiasrv.Client()

	s.srv, err = service.NewService(&service.Config{
		Repository: s.repo,
		Connections: connections.Connections{
			Invitation:   s.invitation,
			Listing:      s.listing,
			Offer:        s.offer,
			Notification: s.notification,
			Document:     s.document,
			Closecom:     s.closecom,
			Viewing:      s.viewing,
		},
		Wistia: service.Wistia{
			URL:               wistiaurl,
			AccessToken:       "abc",
			WebinarsProjectID: 114514,
			HowToProjectID:    191919,
			HTTPClient:        wistiaclient,
		},
		ZeebeClient: s.zbcclient,
	})
	s.Require().NoError(err)

	s.closecom.EXPECT().LeadOpportunitySave(gomock.Any(), gomock.Any()).
		Return(&closecom.LeadOpportunitySaveResponse{}, nil).
		MinTimes(0)
}

func (s *serviceSuite) TearDownTest() {
	s.ctrl.Finish()

	s.testCancel()
}

func (s *serviceSuite) TearSuite() {
	s.rootCancel()
}

func TestService(t *testing.T) {
	suite.Run(t, &serviceSuite{})
}
