package service

import (
	"context"
	"fmt"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
	"gitlab.mashroom.com/mashroom/backend/account/internal/util"
	"gitlab.mashroom.com/mashroom/backend/common/generic/log"
	genericUtil "gitlab.mashroom.com/mashroom/backend/common/generic/util"
	invitation "gitlab.mashroom.com/mashroom/backend/invitation/api"
	offer "gitlab.mashroom.com/mashroom/backend/offer/api"
	"google.golang.org/protobuf/types/known/wrapperspb"
)

func (srv *serviceImpl) updateAccountDetails(m *model.Account, acc *account.Account, needUpdateOffers *bool) {
	val := acc.GetDetailsValue()
	mod := &m.Details

	nameChanged := false

	if val.GetFirstName() != nil {
		mod.FirstName = val.GetFirstNameValue()
		*needUpdateOffers = true
		nameChanged = true
	}

	if val.GetLastName() != nil {
		mod.LastName = val.GetLastNameValue()
		*needUpdateOffers = true
		nameChanged = true
	}

	if val.GetBirthday() != nil {
		mod.Birthday = val.GetBirthdayValue().AsTime()
	}

	if val.GetAvatar() != nil {
		mod.Avatar = val.GetAvatarValue()
	}

	if val.GetGender() != nil {
		mod.Gender = val.GetGenderValue()
	}

	if val.GetUkNi() != nil {
		mod.UkNi = val.GetUkNiValue()
	}

	if nameChanged {
		mod.FullName = fmt.Sprintf("%s %s", mod.FirstName, mod.LastName)
	}
}

func (srv *serviceImpl) updateAccountLocation(m *model.Account, acc *account.Account, needUpdateOffers *bool) {
	val := acc.GetLocationValue()
	mod := &m.Location

	if val.GetPostcode() != nil {
		mod.Postcode = val.GetPostcodeValue()
		*needUpdateOffers = true
	}

	if val.GetAddress1() != nil {
		mod.Address1 = val.GetAddress1Value()
		*needUpdateOffers = true
	}

	if val.GetAddress2() != nil {
		mod.Address2 = val.GetAddress2Value()
		*needUpdateOffers = true
	}

	if val.GetAddress3() != nil {
		mod.Address3 = val.GetAddress3Value()
		*needUpdateOffers = true
	}

	if val.GetAddress4() != nil {
		mod.Address4 = val.GetAddress4Value()
		*needUpdateOffers = true
	}

	if val.GetFlat() != nil {
		mod.Flat = val.GetFlatValue()
		*needUpdateOffers = true
	}

	if val.GetBuilding() != nil {
		mod.Building = val.GetBuildingValue()
		*needUpdateOffers = true
	}

	if val.GetStreet() != nil {
		mod.Street = val.GetStreetValue()
		*needUpdateOffers = true
	}

	if val.GetCity() != nil {
		mod.City = val.GetCityValue()
		*needUpdateOffers = true
	}

	if val.GetCountry() != nil {
		mod.Country = val.GetCountryValue()
		*needUpdateOffers = true
	}

	if val.GetGps() != nil {
		mod.GPS = model.GpsCoordinates{
			Longitude: val.GetGpsValue().GetLongitude(),
			Latitude:  val.GetGpsValue().GetLatitude(),
		}
		*needUpdateOffers = true
	}
}

func (srv *serviceImpl) updateAccountContact(m *model.Account, acc *account.Account, needUpdateOffers *bool) {
	val := acc.GetContactValue()
	mod := &m.Contact

	if val.GetEmailPrimary() != nil {
		mod.EmailPrimary = val.GetEmailPrimaryValue()
		*needUpdateOffers = true
	}

	if val.GetPhonePrimary() != nil {
		mod.PhonePrimary = util.NormalizePhoneNumber(val.GetPhonePrimaryValue())
		*needUpdateOffers = true
	}

	if val.GetPhonesSecondary() != nil {
		arr := val.GetPhonesSecondaryValue()

		var phones []*model.Phone

		for _, p := range arr.GetValue() {
			phones = append(phones, &model.Phone{
				Number: util.NormalizePhoneNumber(p.GetNumber()),
				Type:   p.GetType(),
			})
		}

		mod.PhonesSecondary = phones
	}

	if val.GetContactStatus() != nil {
		if val.GetContactStatusValue().GetEmailVerified() != nil {
			mod.ContactStatus.EmailVerified = val.GetContactStatusValue().GetEmailVerifiedValue()
		}

		if val.GetContactStatusValue().GetPhoneVerified() != nil {
			mod.ContactStatus.PhoneVerified = val.GetContactStatusValue().GetPhoneVerifiedValue()
		}
	}
}

func (srv *serviceImpl) updateAccountBank(m *model.Account, acc *account.Account) (needNotifyBankDetailsChanged bool) {
	val := acc.GetBankDetailsValue()
	mod := &m.BankDetails

	if acc.GetBankDetails() != nil {
		mod.AccountNumber = val.GetAccountNumber()
		mod.AccountName = val.GetAccountName()
		mod.SortCode = val.GetSortCode()
		mod.Society = val.GetSociety()
		mod.PersonalReference = val.GetPersonalReference()

		needNotifyBankDetailsChanged = true
	}

	return
}

func (srv *serviceImpl) updateAccount(
	m *model.Account,
	acc *account.Account,
	trusted bool,
) (needUpdateOffers, needNotifyBankDetailsChanged bool) {
	srv.updateAccountDetails(m, acc, &needUpdateOffers)

	srv.updateAccountLocation(m, acc, &needUpdateOffers)

	srv.updateAccountContact(m, acc, &needUpdateOffers)

	if acc.EnableLandlordTool != nil {
		m.EnableLandlordTool = acc.GetEnableLandlordToolValue()
	}

	if trusted {
		needNotifyBankDetailsChanged = srv.updateAccountBank(m, acc)

		if acc.GetAdmin() != nil {
			m.Admin = acc.GetAdminValue()
		}

		if acc.GetBlock() != nil {
			m.Block = acc.GetBlockValue()
		}

		if acc.GetTest() != nil {
			m.Test = acc.GetTestValue()
		}
	}

	if m.EntryPoint == 0 && acc.Entrypoint != 0 {
		m.EntryPoint = acc.Entrypoint
	}

	return
}

func (srv *serviceImpl) updateInvitations(ctx context.Context, acc *model.Account) error {
	if acc.Admin {
		return nil
	}

	_, err := srv.config.Connections.Invitation.InvitationUpdateInviteeID(
		ctx,
		&invitation.InvitationUpdateInviteeIDRequest{
			Email:     acc.Contact.EmailPrimary,
			InviteeId: acc.ID,
		},
	)
	if err != nil {
		log.WithError(err).Infof("updating user ID for invitations")

		return err
	}

	return nil
}

func (srv *serviceImpl) updateOffers(ctx context.Context, acc *model.Account) error {
	listRequest := &offer.ListRequest{
		User: acc.ID,
		States: &offer.ListRequest_StatesValue{
			StatesValue: &offer.OfferStateArray{
				Value: []offer.OfferState{
					offer.OfferState_PENDING,
					offer.OfferState_ACTIVE,
					offer.OfferState_ACCEPTED,
					offer.OfferState_HOLDING_DEPOSIT,
					offer.OfferState_AWAITING_CONFIRM,
					offer.OfferState_DEPOSIT_CONFIRMED,
					offer.OfferState_AWAITING_APPLICANT_CHECK_PAYMENT,
					offer.OfferState_CHECKING,
					offer.OfferState_AWAITING_CHECKS,
					offer.OfferState_AWAITING_GUARANTOR,
					offer.OfferState_CONTRACT_REVIEW_AND_CLAUSES,
				},
			},
		},
	}

	incoming, err := srv.config.Connections.Offer.ListIncoming(ctx, listRequest)
	if err != nil {
		log.WithError(err).Errorf("getting incoming offers for account %v", acc.ID)
	}

	for _, o := range incoming.GetOffers() {
		_, err = srv.config.Connections.Offer.ServiceUpdate(ctx, &offer.ServiceUpdateRequest{
			Offer: &offer.Offer{
				Id: o.Id,
				Landlord: &offer.Landlord{
					BillingAddress: wrapperspb.String(getBillingAddressFromLocation(acc)),
					FirstName:      wrapperspb.String(acc.Details.FirstName),
					LastName:       wrapperspb.String(acc.Details.LastName),
					EmailPrimary:   wrapperspb.String(acc.Contact.EmailPrimary),
					PhonePrimary:   wrapperspb.String(acc.Contact.PhonePrimary),
				},
			},
		})
		if err != nil {
			log.WithError(err).Error("Error update incoming offers")
			return err
		}
	}

	outgoing, err := srv.config.Connections.Offer.ListOutgoing(ctx, listRequest)
	if err != nil {
		log.WithError(err).Errorf("getting incoming offers for account %v", acc.ID)
	}

	for _, o := range outgoing.GetOffers() {
		tenants := o.GetTenants().GetValue()

		if len(tenants) == 0 || tenants[0] == nil {
			continue
		}

		tenants[0].BillingAddress = wrapperspb.String(getBillingAddressFromLocation(acc))
		tenants[0].FirstName = wrapperspb.String(acc.Details.FirstName)
		tenants[0].LastName = wrapperspb.String(acc.Details.LastName)
		tenants[0].EmailPrimary = wrapperspb.String(acc.Contact.EmailPrimary)
		tenants[0].PhonePrimary = wrapperspb.String(acc.Contact.PhonePrimary)

		_, err = srv.config.Connections.Offer.ServiceUpdate(ctx, &offer.ServiceUpdateRequest{
			Offer: &offer.Offer{
				Id: o.Id,
				Tenants: &offer.TenantArrayValue{
					Value: tenants,
				},
			},
		})
		if err != nil {
			log.WithError(err).Error("Error update outgoing offer")
			return err
		}
	}

	return nil
}

func getBillingAddressFromLocation(acc *model.Account) string {
	loc := &acc.Location

	return genericUtil.JoinNonblankStrings(
		" ",
		loc.Address1,
		loc.Address2,
		loc.City,
		loc.Country,
		loc.Postcode,
	)
}
