package service

import (
	"net/mail"
	"regexp"

	account "gitlab.mashroom.com/mashroom/backend/account/api"
	"gitlab.mashroom.com/mashroom/backend/account/internal/model"
)

func validateEach(
	valid bool,
	total int,
	f func(i int, valid bool) (bool, []account.AccountError),
) (bool, []*account.AccountValidation) {
	var all []*account.AccountValidation

	for i := 0; i < total; i++ {
		var errs []account.AccountError

		valid, errs = f(i, valid)

		if len(all) > 0 {
			all = append(all, &account.AccountValidation{
				Errors: errs,
			})
		} else {
			all = append(all, nil)
		}
	}

	return valid, all
}

func validateList(
	valid, cond bool,
	curr []account.AccountError,
	errs ...account.AccountError,
) (bool, []account.AccountError) {
	if cond {
		return valid, curr
	}

	return false, append(curr, errs...)
}

func validate(valid, cond bool, errs ...account.AccountError) (bool, *account.AccountValidation) {
	if cond {
		return valid, nil
	}

	return false, &account.AccountValidation{
		Errors: errs,
	}
}

func (srv *serviceImpl) validateAccountDetails(
	valid bool,
	m *model.Account,
	acc *account.Account,
) (bool, *account.AccountDetailsVerification) {
	if acc.GetDetails() == nil {
		return valid, nil
	}

	mod := m.Details
	verif := &account.AccountDetailsVerification{}

	if acc.GetDetails() != nil {
		valid, verif.FirstName = validate(
			valid,
			len(mod.FirstName) > 0,
			account.AccountError_ACCOUNT_ERROR_INVALID_NAME,
		)

		valid, verif.LastName = validate(
			valid,
			len(mod.LastName) > 0,
			account.AccountError_ACCOUNT_ERROR_INVALID_NAME,
		)
	}

	return valid, verif
}

func (srv *serviceImpl) validateAccountLocation(
	valid bool,
	m *model.Account,
	acc *account.Account,
) (bool, *account.AccountLocationVerification) {
	if acc.GetLocation() == nil {
		return valid, nil
	}

	mod := m.Location
	verif := &account.AccountLocationVerification{}

	if acc.GetLocationValue().GetPostcode() != nil {
		valid, verif.Postcode = validate(
			valid,
			len(mod.Postcode) > 0,
			account.AccountError_ACCOUNT_ERROR_INVALID_LOCATION,
		)
	}

	return valid, verif
}

var emailTLDRegex = regexp.MustCompile(`[^@]@.*[^.]\.[^.]`)

func (srv *serviceImpl) validateAccountContact(
	valid bool,
	m *model.Account,
	acc *account.Account,
) (bool, *account.AccountContactVerification) {
	if acc.GetContact() == nil {
		return valid, nil
	}

	mod := m.Contact
	verif := &account.AccountContactVerification{}

	if acc.GetContactValue().GetEmailPrimary() != nil {
		addr, err := mail.ParseAddress(mod.EmailPrimary)

		// hostname-style and TLD-based emails are though perfectly valid, are not supported due to rarity and
		// frequency of user errors with conventional 2nd-level domains.
		valid, verif.EmailPrimary = validate(
			valid,
			len(mod.EmailPrimary) > 0 && err == nil &&
				addr.Address == mod.EmailPrimary && emailTLDRegex.MatchString(mod.EmailPrimary),
			account.AccountError_ACCOUNT_ERROR_INVALID_EMAIL,
		)
	}

	if acc.GetContactValue().GetPhonesSecondaryValue() != nil {
		valid, verif.PhonesSecondary = validateEach(
			valid,
			len(mod.PhonesSecondary),
			func(i int, valid bool) (bool, []account.AccountError) {
				phone := mod.PhonesSecondary[i]

				var ver []account.AccountError

				valid, ver = validateList(
					valid,
					len(phone.Number) > 0,
					nil,
					account.AccountError_ACCOUNT_ERROR_INVALID_PHONE,
				)

				valid, ver = validateList(
					valid,
					phone.Type != account.AccountPhoneType_UNKNOWN_PHONE_TYPE,
					ver,
					append(ver, account.AccountError_ACCOUNT_ERROR_INVALID_PHONE_TYPE)...,
				)

				return valid, ver
			},
		)
	}

	return valid, verif
}

func (srv *serviceImpl) validateAccount(
	m *model.Account,
	acc *account.Account,
) (valid bool, verif *account.AccountVerification) {
	verif = &account.AccountVerification{}
	valid = true

	valid, verif.Details = srv.validateAccountDetails(valid, m, acc)

	valid, verif.Location = srv.validateAccountLocation(valid, m, acc)

	valid, verif.Contact = srv.validateAccountContact(valid, m, acc)

	return
}
